PrimeUIOptions = class(Turbine.UI.Control)
function PrimeUIOptions:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self);
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0);
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    self.notChanging = false

    self.BorderBackground = Turbine.UI.Control()
    self.BorderBackground:SetParent(self)
    self.BorderBackground:SetPosition(10 - 2, 10 - 2)
    self.BorderBackground:SetSize(730 + 4, 450 + 4)
    self.BorderBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.Background = Turbine.UI.Control()
    self.Background:SetParent(self)
    self.Background:SetPosition(10, 10)
    self.Background:SetSize(730, 450)
    self.Background:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    --if true then return end

    self.infoWindow = InfoWindow()

    self.treeviewBoxes = {}

    self.Moving = false

    self.optionInfoText = {
        ["Icon"] = "Changes the icon of the UI element. Click on the white box or current icon to bring up the icon selection menu. Alternatively you can enter the ID of an icon.",
        ["Colour"] = "Changes varies colours setting. Each scrollbar represents a different channel of the ARGB system. (Alpha changes the opacity of the colour, a value of 0 renders a colour invisible)",
        ["Font"] = "Set the font for a UI element. Some UI element will scale the font while others do not. (FixedSys15 is the only always working monospaced font)",
        ["Timer_Stack_Direction"] = "Determines which way timers are added and sorted to a timergroup. Addition will work from the opposite side of the direction.",
        ["Timer_Sorting"] = "Determines which sorting is used for a timergroup.",
        ["TextAlignment"] = "Determines the alignment of the text within the current UI element.",
        ["StackTextAlignment"] = "Determines the alignment of the text within the current UI element.",
        ["Alignment"] = "Sets the alignment of the current UI element in respect to the parent PrimeUI.",

        -- Number
        ["Height"] = "Sets the height of current UI element if applicable",
        ["Width"] = "Sets the width of current UI element if applicable",
        ["OffsetAbove"] = "Sets the offset above this UI element with the above UI element within the PrimeUI or the top of the parent UI.",
        ["OffsetBelow"] = "Sets the offset below this UI element with the below UI element within the PrimeUI or the bottom of the parent UI.",
        ["Scale"] = "Sets the scaling factor of the current UI element or the UI elements within/created.",
        ["X"] = "Sets the X coordinate (only applicable if in advanced mode)",
        ["Y"] = "Sets the Y coordinate (only applicable if in advanced mode)",
        ["BorderSize"] = "The width of the border around the UI element.",
        ["Intervals"] = "Determines the amount of bars that will be drawn over the bar.",
        ["BoxOffset"] = "Distance between the boxes.",
        ["Visibility_Threshold"] = "Determines at what number a timer has to be in order to be visible.",

        ["Text"] = "Changes the text of the current UI element.",

        -- Setting
        ["AllEffects"] = "Turning this setting on will forego specific trigger settings for effects but instead process any effect.",
        ["OnlyBuffs"] = "Only allows buffs to go through selection.",
        ["OnlyDebuffs"] = "Only allows debuffs to go through selection.",
        ["AllHighlight"] = "Highlights every element within this UI element without the need to specify this for these specific elements",
        ["Reset_On_Retarget"] = "This setting will wipe the buffbar when a target swap event is detected.",
        ["AlwaysVisible"] = "Will always turn this UI element visible.",
        ["Clockwise"] = "This setting will turn the visual timer direction.",
        ["GrayOut"] = "Will slightly grayout the timericon.",
        ["Hide_While_Not_Active"] = "Will hide this element when there is no timer active.",
        ["Highlight"] = "Enabling this will highlight this element.",
        ["HighlightWhileOff"] = "Enabling this will highlight this element when it is considered \"off\".",
        ["ShowText"]  = "This setting will toggle the visiblity of the text.",
        ["Visual_Timer"] = "This setting will enable the visual timer for this element.",
        ["Static_Timers"] = "This setting will cause every trigger event associated with this UI element to create a timer that is always visible within the group timer UI.",
        ["Full_Border"] = "This setting will cause the UI element to draw a full border around the innerbox.",
        ["RKLayout"] = "This setting will cause the UI element to go up to only 10 charges instead of 20.",
        ["Conjuction"] = "This will cause the icon to show up if ALL conditions are satisfied. If this setting is turned off the icon will show up if ANY condition is satisfied.",

        -- Triggers
        ["Effects"] = "This places a trigger on any effect added that matches the conditions specified. This trigger type will keep track when an effect is added and removed and update the UI accordingly.",
        --["Skills"] = "Whenever a skill matching ",
        ["PlayerHits"] = "Whenever the player hits any target with a hit which name matches the conditions specified it utilizes the function of the UI.",
        ["EnemyHits"] = "Whenever the player is hit by a hit which name matches the conditions specified it utilizes the function of the UI.",
        ["EffectAdded"] = "Whenever an effect is added to the player that matches the conditions specified it utilizes the function of the UI.",
        ["EffectRemoved"] = "Whenever an effect is removed from the player that matches the conditions specified it utilizes the function of the UI.",
        ["PartyChat"] = "When a phrase is detected in party chat (fellowship or raid) it utilizes the function of the UI. (Markup colours have to be added to the matching string)",
        ["StandardChat"] = "When a phrase is detected in standard chat it utilizes the function of the UI. (Markup colours have to be added to the matching string)",
        ["SayChat"] = "When a phrase is detected in say chat it utilizes the function of the UI. (Markup colours have to be added to the matching string)",
        ["HealthThreshold"] = "When a player falls below or moves above a set threshold it utilizes the function of the UI. (This effect has a cooldown)",
        ["EffectAdded_Group"] = "Whenever an effect is added to any player in the group (does not include the player) that matches the conditions specified it utilizes the function of the UI.",
        ["EffectAdded_Target"] = "Whenever an effect is added to the target of the player that matches the conditions specified it utilizes the function of the UI.",
        ["Target_HealthThreshold"] = "When a target falls below or moves above a set threshold it utilizes the function of the UI. (This effect has a cooldown)",

    }

    self.textAlignments = {
        ["BottomCenter"] = Turbine.UI.ContentAlignment.BottomCenter,
        ["BottomLeft"] = Turbine.UI.ContentAlignment.BottomLeft,
        ["BottomRight"]	= Turbine.UI.ContentAlignment.BottomRight,
        ["MiddleCenter"] = Turbine.UI.ContentAlignment.MiddleCenter,
        ["MiddleLeft"] = Turbine.UI.ContentAlignment.MiddleLeft,
        ["MiddleRight"] = Turbine.UI.ContentAlignment.MiddleRight,
        ["TopCenter"] = Turbine.UI.ContentAlignment.TopCenter,
        ["TopLeft"] = Turbine.UI.ContentAlignment.TopLeft,
        ["TopRight"] = Turbine.UI.ContentAlignment.TopRight,
    }

    self.timerGroupAlignments = {
        ["Alphabetical"] = true,
        ["Alphabetical Reverse"] = true,
        ["Hight to Low (Remaining duration)"] = true,
        ["Low to High (Remaining duration)"] = true,
        ["Hight to Low (Max duration)"] = true,
        ["Low to High (Max duration)"] = true,
    }
    self.timeStackAlignments = {
        ["Up"] = true,
        ["Down"] = true,
    }

    self.Classes = {
        ["Beorning"] = Turbine.Gameplay.Class.Beorning,
        ["Brawler"] = Turbine.Gameplay.Class.Brawler,
        ["Burglar"] = Turbine.Gameplay.Class.Burglar,
        ["Captain"] = Turbine.Gameplay.Class.Captain,
        ["Champion"] = Turbine.Gameplay.Class.Champion,
        ["Guardian"] = Turbine.Gameplay.Class.Guardian,
        ["Hunter"] = Turbine.Gameplay.Class.Hunter,
        ["LoreMaster"] = Turbine.Gameplay.Class.LoreMaster,
        ["Minstrel"] = Turbine.Gameplay.Class.Minstrel,
        ["RuneKeeper"] = Turbine.Gameplay.Class.RuneKeeper,
        ["Warden"] = Turbine.Gameplay.Class.Warden,
        ["BlackArrow"] = Turbine.Gameplay.Class.BlackArrow,
        ["Defiler"] = Turbine.Gameplay.Class.Defiler,
        ["Reaver"] = Turbine.Gameplay.Class.Reaver,
        ["Stalker"] = Turbine.Gameplay.Class.Stalker,
        ["WarLeader"] = Turbine.Gameplay.Class.WarLeader,
        ["Weaver"] = Turbine.Gameplay.Class.Weaver
    }

    self.alignments = {
        ["Left"] = true,
        ["Middle"] = true,
        ["Right"] = true,
    }

    self.UIElements = {
        ["Icon"] = true,
        ["Text"] = true,
        ["TimerIcon"] = true,

        ["MoraleBar"] = true,
        ["MoraleBar_Target"] = true,
        ["PowerBar_Target"] = true,
        ["PowerBar"] = true,
        ["WrathBar"] = true,

        ["ResourceBar"] = true,
        
        ["BuffBar"] = true,
        ["BuffBar_Target"] = true,

        ["Skills"] = true,

        ["ValueBar"] = true,
        ["Timer"] = true,
        ["TimerGroup"] = true,

        ["Countdown"] = true,
        ["Sound"] = true,

        ["SkillLabel"] = true,
    }

    self.SortOrderTable = {
        [1] = "Text",
        [2] = "Text",
        [10] = "Number",
        [20] = "Alignment",
        [21] = "Alignment",
        [22] = "Alignment",
        [23] = "Alignment",
        [24] = "Font",
        [30] = "Setting",
        [35] = "Icon",
        [40] = "Colour",
        [50] = "Trigger",
    }

    --self.iconWindow = IconOverview()
    --self.effectWindow = HistoricalEffectOverview()
    --self.skillWindow = SkillOverview()
    --self.skillCodesWindow = SkillOverview2()

    self.IOWindow = IOWindow(self)
    self.NewWindow = NewWindow(self)

    self.leftBorder1 = Turbine.UI.Control()
    self.leftBorder1:SetParent(self)
    self.leftBorder1:SetSize(6, 450)
    self.leftBorder1:SetPosition(204, 10)
    self.leftBorder1:SetMouseVisible(false)
    self.leftBorder1:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.leftBorder1:SetBackColor(Turbine.UI.Color(0.6, 0.125, 0.125, 0.125))

    self.leftBorder2 = Turbine.UI.Control()
    self.leftBorder2:SetParent(self)
    self.leftBorder2:SetSize(3, 450)
    self.leftBorder2:SetPosition(204, 10)
    self.leftBorder2:SetMouseVisible(false)
    self.leftBorder2:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.leftBorder2:SetBackColor(Turbine.UI.Color(0.6, 1, 1, 1))

    self.InfoBorderContainer = Turbine.UI.Control()
    self.InfoBorderContainer:SetParent(self)
    self.InfoBorderContainer:SetPosition(210, 10)
    self.InfoBorderContainer:SetSize(530, 100)
    self.InfoBorderContainer:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.InfoContainer = Turbine.UI.Control()
    self.InfoContainer:SetParent(self)
    self.InfoContainer:SetPosition(210, 10)
    self.InfoContainer:SetSize(530, 100 - 2)
    self.InfoContainer:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    self.InfoHeaderBackground = Turbine.UI.Label()
    self.InfoHeaderBackground:SetParent(self.InfoContainer)
    self.InfoHeaderBackground:SetSize(402, 52)
    self.InfoHeaderBackground:SetMouseVisible(false)
    self.InfoHeaderBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.InfoHeaderLabel = Turbine.UI.Label()
    self.InfoHeaderLabel:SetParent(self.InfoContainer)
    self.InfoHeaderLabel:SetSize(400, 50)
    self.InfoHeaderLabel:SetMouseVisible(false)
    --self.InfoHeaderLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.InfoHeaderLabel:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
    self.InfoHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.InfoHeaderLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold36)
    self.InfoHeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.InfoHeaderLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.InfoHeaderLabel:SetText("PrimeUI Options")

    self.InstanceButton = Turbine.UI.Lotro.Button()
	self.InstanceButton:SetParent(self.InfoContainer)
	self.InstanceButton:SetSize(100, 20)
	self.InstanceButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.InstanceButton:SetPosition(0, 55)
    self.InstanceButton:SetText("Instance")

    self.InstanceMenu = Turbine.UI.ContextMenu()
    self.InstanceMenuItems = self.InstanceMenu:GetItems()
    for key, value in spairs(InstanceController:GetDungeonNameTable(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.InstanceMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.InstanceButton:SetText(value)
            self.currentUI["InstanceLoad"] = value
            PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
        end
    end

    self.InstanceButton.Click = function()
        if self.currentUI then
            self.InstanceMenu:ShowMenu()
        end
    end

    self.RoleButton = Turbine.UI.Lotro.Button()
	self.RoleButton:SetParent(self.InfoContainer)
	self.RoleButton:SetSize(100, 20)
	self.RoleButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.RoleButton:SetPosition(100, 55)
    self.RoleButton:SetText("Role")

    self.RoleMenu = Turbine.UI.ContextMenu()
    self.RoleMenuItems = self.RoleMenu:GetItems()

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.RoleMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.RoleButton:SetText("All")
        self.currentUI["RoleLoad"] = "All"
        PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
    end

    for key, value in spairs(Roles:GetAvailableRoles(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.RoleMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.RoleButton:SetText(value)
            self.currentUI["RoleLoad"] = value
            PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
        end
    end


    self.RoleButton.Click = function()
        if self.currentUI then
            self.RoleMenu:ShowMenu()
        end
    end

    self.LineButton = Turbine.UI.Lotro.Button()
	self.LineButton:SetParent(self.InfoContainer)
	self.LineButton:SetSize(100, 20)
	self.LineButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.LineButton:SetPosition(200, 55)
    self.LineButton:SetText("Line")

    local classMenu = Turbine.UI.ContextMenu()
    local classMenuItems = classMenu:GetItems()

    local classItem = Turbine.UI.MenuItem( "All" )
    classItem.Click = function()
        self.LineButton:SetText("All")
        self.currentUI["LineLoad"] = "All"
        PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
    end
    classMenuItems:Add(classItem)

    local classItem = Turbine.UI.MenuItem( "Blue" )
    classItem.Click = function()
        self.LineButton:SetText("Blue" )
        self.currentUI["LineLoad"] ="Blue"
        PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
    end
    classMenuItems:Add(classItem)

    local classItem = Turbine.UI.MenuItem( "Red" )
    classItem.Click = function()
        self.LineButton:SetText("Red")
        self.currentUI["LineLoad"] ="Red"
        PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
    end
    classMenuItems:Add(classItem)

    local classItem = Turbine.UI.MenuItem( "Yellow" )
    classItem.Click = function()
        self.LineButton:SetText("Yellow")
        self.currentUI["LineLoad"] = "Yellow"
        PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
    end
    classMenuItems:Add(classItem)

    self.LineButton.Click = function()
        if self.currentUI then
            classMenu:ShowMenu()
        end
    end

    self.CombatLabel = Turbine.UI.Label()
	self.CombatLabel:SetParent(self.InfoContainer)
	self.CombatLabel:SetSize(50, 20)
	self.CombatLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.CombatLabel:SetPosition(150, 80)
    self.CombatLabel:SetText("Show")

    self.ICheckBox = Turbine.UI.Lotro.CheckBox()
	self.ICheckBox:SetParent(self.InfoContainer)
	self.ICheckBox:SetSize(250, 20)
	self.ICheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.ICheckBox:SetPosition(190, 80)
    self.ICheckBox:SetText("In combat")
    self.ICheckBox:SetChecked(false)

    self.ICheckBox.CheckedChanged = function()
        if self.currentUI and self.notChanging then
            self.currentUI["ICLoad"] = self.ICheckBox:IsChecked()
            PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
        end
    end

    self.ClassButton = Turbine.UI.Lotro.Button()
	self.ClassButton:SetParent(self.InfoContainer)
	self.ClassButton:SetSize(100, 20)
	self.ClassButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.ClassButton:SetPosition(300, 55)
    self.ClassButton:SetText("Class")

    self.ClassMenu = Turbine.UI.ContextMenu()
    self.ClassMenuItems = self.ClassMenu:GetItems()

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.ClassMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.ClassButton:SetText("All")
        self.currentUI["ClassLoad"] = "All"
        PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
    end

    for key, value in spairs(self.Classes, function(t, a, b) return a < b end) do
        local menuItem = Turbine.UI.MenuItem( key )
        self.ClassMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.ClassButton:SetText(key)
            self.currentUI["ClassLoad"] = key
            PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
        end
    end

    self.ClassButton.Click = function()
        if self.currentUI then
            self.ClassMenu:ShowMenu()
        end
    end

    self.OOCheckBox = Turbine.UI.Lotro.CheckBox()
	self.OOCheckBox:SetParent(self.InfoContainer)
	self.OOCheckBox:SetSize(250, 20)
	self.OOCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.OOCheckBox:SetPosition(280, 80)
    self.OOCheckBox:SetText("Out of combat")
    self.OOCheckBox:SetChecked(false)

    self.OOCheckBox.CheckedChanged = function()
        if self.currentUI and self.notChanging then
            self.currentUI["OOCLoad"] = self.OOCheckBox:IsChecked()
            PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
        end
    end



    self.NeverLoadCheckbox = Turbine.UI.Lotro.CheckBox()
	self.NeverLoadCheckbox:SetParent(self.InfoContainer)
	self.NeverLoadCheckbox:SetSize(150, 20)
	self.NeverLoadCheckbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.NeverLoadCheckbox:SetPosition(420, 60)
    self.NeverLoadCheckbox:SetText("Never load")
    self.NeverLoadCheckbox:SetChecked(false)

    self.NeverLoadCheckbox.CheckedChanged = function()
        if self.currentUI and self.notChanging then
            self.currentUI["NeverLoad"] = self.NeverLoadCheckbox:IsChecked()
            PrimeUIHandlerObject:ChangeUIMetaData(self.currentUI)
        end
    end


    self.OutputButton = Turbine.UI.Lotro.Button()
	self.OutputButton:SetParent(self.InfoContainer)
	self.OutputButton:SetSize(125, 40)
	self.OutputButton:SetPosition(405, 0)
    self.OutputButton:SetText("Output PrimeUI")

    self.OutputButton.Click = function()
        if self.currentUI and self.notChanging then
            self.IOWindow:View(self.currentUI)
        end
    end

    self.SetupContainer = Turbine.UI.Control()
    self.SetupContainer:SetParent(self)
    self.SetupContainer:SetPosition(210, 110)
    self.SetupContainer:SetSize(530 - 10, 350)
    self.SetupContainer:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    self.CurrentLeft = Turbine.UI.Control()
    self.CurrentLeft:SetParent(self.Background)
    self.CurrentLeft:SetPosition(0, 20)
    self.CurrentLeft:SetSize(22, 26)
    self.CurrentLeft:SetBackground(0x411BAF1E)
    self.CurrentLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.CurrentRight = Turbine.UI.Control()
    self.CurrentRight:SetParent(self.Background)
    self.CurrentRight:SetPosition(195 - 23, 20)
    self.CurrentRight:SetSize(23, 26)
    self.CurrentRight:SetBackground(0x411BAF20)
    self.CurrentRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.CurrentMiddle = Turbine.UI.Control()
    self.CurrentMiddle:SetParent(self.Background)
    self.CurrentMiddle:SetPosition(22, 20)
    self.CurrentMiddle:SetSize(195 - 40, 26)
    self.CurrentMiddle:SetBackground(0x411BAF19)
    self.CurrentMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.CurrentLabel = Turbine.UI.Label()
    self.CurrentLabel:SetParent(self.Background)
    self.CurrentLabel:SetPosition(6, 23)
    self.CurrentLabel:SetSize(195, 20)
    self.CurrentLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    self.CurrentLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    --self.CurrentLabel:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))
    self.CurrentLabel:SetText("PrimeUIs")

    self.PrimeUIContainer = Turbine.UI.TreeView()
    self.PrimeUIContainer:SetParent(self.Background)
    self.PrimeUIContainer:SetSize(200 - 10, self.Background:GetHeight() - 46)
    self.PrimeUIContainer:SetPosition(0, 46)
    self.PrimeUIContainer:SetVisible(true)
    self.PrimeUIContainer:SetWantsKeyEvents(true)
    self.PrimeUIContainer:SetMouseVisible(true)
    self.PrimeUIContainer:SetIndentationWidth( 0 );

    self.PrimeUIContainerScrollbar = Turbine.UI.Lotro.ScrollBar();
    self.PrimeUIContainerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical );
    self.PrimeUIContainerScrollbar:SetParent( self.Background );
    self.PrimeUIContainerScrollbar:SetPosition( 200 - 10, 46 );
    self.PrimeUIContainerScrollbar:SetSize( 10, self.Background:GetHeight() - 46 );
    self.PrimeUIContainer:SetVerticalScrollBar( self.PrimeUIContainerScrollbar );

    self.ImportButton = Turbine.UI.Lotro.Button()
	self.ImportButton:SetParent(self.Background)
	self.ImportButton:SetSize(115, 40)
	self.ImportButton:SetPosition(0, 0)
    self.ImportButton:SetText("Import PrimeUI")

    self.ImportButton.Click = function()
        self.IOWindow:View()
    end

    self.rootNodes = self.PrimeUIContainer:GetNodes();

    self.boxes = {}
    self.data = PrimeUIHandlerObject:GetData()

    for _, data in spairs(self.data, function(t, a, b) return t[a]["Name"] < t[b]["Name"] end) do
        self.boxes[data["Name"]] = self:CreateUIItem(self.SetupContainer, data)
    end

    self.UIElementsMenu = Turbine.UI.ContextMenu()
    self.UIElementsMenuItems = self.UIElementsMenu:GetItems()

    local sortedUIElements = {}
    for key, value in pairs(self.UIElements) do table.insert( sortedUIElements, key ) end
    table.sort( sortedUIElements )

    self.AddButton = Turbine.UI.Lotro.Button()
	self.AddButton:SetParent(self.InfoContainer)
	self.AddButton:SetSize(150, 20)
	self.AddButton:SetPosition(0, 80)
    self.AddButton:SetText("Add UI Element")

    for key, name in pairs(sortedUIElements) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            if self.currentUI then
                self.boxes[self.currentUI["Name"]]:SetParent(nil)
                self.boxes[self.currentUI["Name"]] = nil
                local count = tablelength(self.currentUI["PrimeUI"]) + 1
                local newTable = self:GetUIElementData(name, count)
                table.insert(self.currentUI["PrimeUI"], newTable)
                PrimeUIHandlerObject:Refresh(self.currentUI["Name"], self.currentUI["PrimeUI"])
                self.data = PrimeUIHandlerObject:GetData()
                for _, data in pairs(self.data) do
                    if not(self.boxes[data["Name"]]) then
                        self.boxes[data["Name"]] = self:CreateUIItem(self.SetupContainer, data)
                    end
                end
                self.boxes[self.currentUI["Name"]]:SetVisible(true)
            end
        end
        
        self.UIElementsMenuItems:Add(menuItem)
    end

    self.AddButton.Click = function()
        if self.currentUI then
            self.UIElementsMenu:ShowMenu()
        end
    end

    self.NewButton = Turbine.UI.Lotro.Button()
	self.NewButton:SetParent(self.Background)
	self.NewButton:SetSize(80, 20)
	self.NewButton:SetPosition(115, 0)
    self.NewButton:SetText("New")

    self.NewButton.Click = function()
        self.NewWindow:View()
    end

    self.MoveButton = Turbine.UI.Lotro.Button()
	self.MoveButton:SetParent(self.InfoContainer)
	self.MoveButton:SetSize(125, 40)
	self.MoveButton:SetPosition(405, 30)
    self.MoveButton:SetText("Move")

    self.MoveButton.Click = function()
        if self.currentUI then
            self.Moving = not(self.Moving)
            PrimeUIHandlerObject:Move(self.currentUI["Name"])
        end
    end

    self.DeleteButton = Turbine.UI.Lotro.Button()
	self.DeleteButton:SetParent(self.InfoContainer)
	self.DeleteButton:SetSize(125, 40)
	self.DeleteButton:SetPosition(400, 80)
    self.DeleteButton:SetText("Delete UI")

    self.DeleteButton.Click = function()
        if self.currentUI then
            PrimeUIHandlerObject:Delete(self.currentUI["Name"])
            self.rootNodes:Remove(self.treeviewBoxes[self.currentUI["Name"]])
            self.treeviewBoxes[self.currentUI["Name"]]:SetParent(nil)
            self.treeviewBoxes[self.currentUI["Name"]] = nil
            self.boxes[self.currentUI["Name"]]:SetVisible(false)
            self.boxes[self.currentUI["Name"]]:SetParent(nil)
            self.boxes[self.currentUI["Name"]] = nil
            self.InfoHeaderLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
            self.InfoHeaderLabel:SetText("PrimeUIs")
            self.currentUI = nil
            self.data = PrimeUIHandlerObject:GetData()
        end
    end
    
end

function PrimeUIOptions:Add(data)
    if not(self.boxes[data["Name"]]) and PrimeUIHandlerObject:AddUI(data) then
        self.boxes[data["Name"]] = self:CreateUIItem(self.SetupContainer, data)
        return true
    else
        ChatOutput("Failed to add PrimeUI: Name already exists")
        return false
    end
end

function PrimeUIOptions:CreateNew(name, advancedMode)
    if PrimeUIHandlerObject:AddUI({
        ["InstanceLoad"] = "All",
        ["RoleLoad"] = "All",
        ["ClassLoad"] = "All",
        ["CharLoad"] = false,
        ["ZoneLoad"] = false,
        ["GroupLoad"] = false,
        ["ClassLoad"] = "All",
        ["LineLoad"] = "All",
        ["ICLoad"] = true,
        ["OOCLoad"] = true,
        ["NeverLoad"] = false,
        ["X"] = 100,
        ["Y"] = 100,
        ["Name"] = name,
        ["PrimeUI"] = {},
        ["AdvancedMode"] = advancedMode
    }) then
        self.data = PrimeUIHandlerObject:GetData()
        for _, data in pairs(self.data) do
            if not(self.boxes[data["Name"]]) then
                self.boxes[data["Name"]] = self:CreateUIItem(self.SetupContainer, data)
            end
        end

        return true
    else
        return false
    end
end

function PrimeUIOptions:ExtendTreeview(data, width)

    if self.treeviewBoxes[data["Name"]] then return end

    local SourceNode = Turbine.UI.TreeNode()
    SourceNode:SetSize(width - 10, 20 + 2)

    SourceNode.SourceLabel = Turbine.UI.Label()
    SourceNode.SourceLabel:SetParent(SourceNode)
    SourceNode.SourceLabel:SetPosition(1,1)
    SourceNode.SourceLabel:SetSize(width - 10- 2, 20)
    SourceNode.SourceLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    SourceNode.SourceLabel:SetMouseVisible(true)

    SourceNode.LefLabel = Turbine.UI.Label()
    SourceNode.LefLabel:SetParent(SourceNode)
    SourceNode.LefLabel:SetPosition(0, 0)
    SourceNode.LefLabel:SetSize(30, 20)
    SourceNode.LefLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.LefLabel:SetBackground(0x4111D1FA)
    SourceNode.LefLabel:SetMouseVisible(false)

    SourceNode.RigthLabel = Turbine.UI.Label()
    SourceNode.RigthLabel:SetParent(SourceNode)
    SourceNode.RigthLabel:SetPosition(width - 157, 0)
    SourceNode.RigthLabel:SetSize(147, 20)
    SourceNode.RigthLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.RigthLabel:SetBackground(0x4111D1FA)
    SourceNode.RigthLabel:SetMouseVisible(false)

    SourceNode.MidLabel = Turbine.UI.Label()
    SourceNode.MidLabel:SetParent(SourceNode)
    SourceNode.MidLabel:SetPosition(30, 0)
    SourceNode.MidLabel:SetSize(width - 70, 20)
    SourceNode.MidLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.MidLabel:SetBackground(0x4111D201)
    SourceNode.MidLabel:SetMouseVisible(false)

    SourceNode.textLabel = Turbine.UI.Label()
    SourceNode.textLabel:SetParent(SourceNode)
    SourceNode.textLabel:SetPosition(1,1)
    SourceNode.textLabel:SetSize(width - 10- 2, 20)
    SourceNode.textLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    SourceNode.textLabel:SetMouseVisible(false)
    SourceNode.textLabel:SetText(data["Name"])

    SourceNode.SourceLabel.MouseEnter = function()
        SourceNode.LefLabel:SetBackground(0x4111D1FB)
        SourceNode.RigthLabel:SetBackground(0x4111D1FB)
        SourceNode.MidLabel:SetBackground(0x4111D202)
    end

    SourceNode.SourceLabel.MouseLeave = function()
        SourceNode.LefLabel:SetBackground(0x4111D1FA)
        SourceNode.RigthLabel:SetBackground(0x4111D1FA)
        SourceNode.MidLabel:SetBackground(0x4111D201)
    end

    SourceNode.SourceLabel.MouseDown = function()
        SourceNode.LefLabel:SetBackground(0x4111D1FC)
        SourceNode.RigthLabel:SetBackground(0x4111D1FC)
        SourceNode.MidLabel:SetBackground(0x4111D203)
    end

    SourceNode.SourceLabel.MouseUp = function()
        SourceNode.LefLabel:SetBackground(0x4111D1FA)
        SourceNode.RigthLabel:SetBackground(0x4111D1FA)
        SourceNode.MidLabel:SetBackground(0x4111D201)
    end

    SourceNode.SourceLabel.MouseClick = function()
        for key, value in pairs(self.boxes) do
            value:SetVisible(false)
        end
        self.boxes[data["Name"]]:SetVisible(true)
        self.notChanging = false
        self:FillInfoContainer(data)
        self.notChanging = true
        if self.currentUI and self.Moving then
            PrimeUIHandlerObject:Move(self.currentUI["Name"])
            self.Moving = false
        end
        self.currentUI = data
    end

    self.treeviewBoxes[data["Name"]] = SourceNode

    self.rootNodes:Add(SourceNode)
end

function PrimeUIOptions:FillInfoContainer(data)
    self.InfoHeaderLabel:SetText(data["Name"])
    self.InstanceButton:SetText(data["InstanceLoad"])
    self.RoleButton:SetText(data["RoleLoad"])
    self.ClassButton:SetText(data["ClassLoad"])
    self.LineButton:SetText(data["LineLoad"])
    self.OOCheckBox:SetChecked(data["OOCLoad"])
    self.ICheckBox:SetChecked(data["ICLoad"])
    self.NeverLoadCheckbox:SetChecked(data["NeverLoad"])
    if data.AdvancedMode then
        self.InfoHeaderLabel:SetOutlineColor(Turbine.UI.Color(0, 0.6, 0))
    else
        self.InfoHeaderLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    end

end

function PrimeUIOptions:CreateUIItem(parent, ObjectData)
    local width = parent:GetWidth() - 10
    local height = parent:GetHeight()

    local data1 = ObjectData["PrimeUI"]
    local name = ObjectData["Name"]

    self:ExtendTreeview(ObjectData, 200)

    local valuebarBackground = Turbine.UI.Control()
    valuebarBackground:SetParent(parent)
    valuebarBackground:SetSize(width + 10, height)
    valuebarBackground:SetVisible(false)
    --valuebarBackground:SetMouseVisible(false)
    --valuebarBackground:SetBackColor(GetRandomColour())

    local valuebar = Turbine.UI.TreeView()
    valuebar:SetParent(valuebarBackground)
    valuebar:SetSize(width, height)
    valuebar:SetIndentationWidth(0)

    local valuebarNodes = valuebar:GetNodes();
    --valuebar:SetMouseVisible(false)

    self:UITreeView(data1, width, valuebarNodes, name)

    valuebarScrollbar = Turbine.UI.Lotro.ScrollBar();
    valuebarScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    valuebarScrollbar:SetParent(valuebarBackground)
    valuebarScrollbar:SetPosition(width, 0)
    valuebarScrollbar:SetSize(10, height)
    valuebar:SetVerticalScrollBar(valuebarScrollbar)

    --valuebar:ExpandAll()

    return valuebarBackground
end

function PrimeUIOptions:GetUIElementData(type, id)
    if type == "Icon" then
        return {
            ["Order"] = id,
            ["Type"] = "Icon",
            ["Height"] = 32,
            ["Width"] = 32,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BackgroundColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},
            ["Alignment"] = "Middle",
            ["TextAlignment"] = "MiddleCenter",
            ["Text"] = "",
            ["Font"] = "TrajanProBold16",
            ["Font_Colour"] = {[1] = 1, [2] = 1, [3] = 1, [4] = 1},
            ["Icon"] = 1090551617,
            ["AlwaysVisible"] = false,
            ["Effects"] = {
            },
            ["Highlight"] = false,
            ["HealthThreshold"] = {}, 
            ["Conjuction"] = true,
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "TimerIcon" then
        return {
            ["Order"] = id,
            ["Type"] = "TimerIcon",
            ["Height"] = 32,
            ["Width"] = 32,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BorderSize"] = 0,
            ["GrayOut"] = false,
            ["HighlightWhileOff"] = false,
            ["Alignment"] = "Middle",
            ["TextAlignment"] = "MiddleCenter",
            ["ShowText"] = true,
            ["Font"] = "TrajanProBold16",
            ["Font_Colour"] = {[1] = 1, [2] = 1, [3] = 1, [4] = 1},
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},
            ["Icon"] = 1090551617,
            ["AlwaysVisible"] = false,
            ["Clockwise"] = true,
            ["Visual_Timer"] = true,
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},
            ["Skills"] = {},
            ["EffectAdded"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectRemoved"] = {},
            ["EffectAdded_Target"] = {},
            ["Highlight"] = false,
            ["Hide_While_Not_Active"] = false,
            ["Visibility_Threshold"] = 9999,        
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "MoraleBar" then
        return {
            ["Order"] = id,
            ["Type"] = "MoraleBar",
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Height"] = 20,
            ["Text"] = "<current>/<total> (<percentage>%)",
            ["Smooth"] = true,
            ["Colours"] = { [1] = {[1] = 0.6, [2] = 0, [3] = 0.5, [4] = 0}, 
                            [2] = {[1] = 0.6, [2] = 0, [3] = 1, [4] = 0}, 
                            [3] = {[1] = 0.6, [2] = 0.7, [3] = 0, [4] = 1}},
            ["Alignment"] = "Middle",
            ["Intervals"] = 0,
            ["Interval_Colour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
            ["Width"] = 500,
            ["Font"] = "BookAntiquaBold19",
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},        
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "MoraleBar_Target" then
        return {
            ["Order"] = id,
            ["Type"] = "MoraleBar_Target",
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Height"] = 20,
            ["Text"] = "<current>/<total> (<percentage>%)",
            ["Smooth"] = false,
            ["Colours"] = { [1] = {[1] = 0.6, [2] = 0, [3] = 0.5, [4] = 0}, 
                            [2] = {[1] = 0.6, [2] = 0, [3] = 1, [4] = 0}, 
                            [3] = {[1] = 0.6, [2] = 0.7, [3] = 0, [4] = 1}},
            ["Alignment"] = "Middle",
            ["Intervals"] = 0,
            ["Interval_Colour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
            ["Width"] = 500,
            ["Font"] = "BookAntiquaBold19",
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},        
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "PowerBar" then
        return {
            ["Order"] = id,
            ["Type"] = "PowerBar",
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Height"] = 20,
            ["Text"] = "<current>/<total> (<percentage>%)",
            ["Smooth"] = true,
            ["Alignment"] = "Middle",
            ["Colours"] = { [1] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
                            [2] = {[1] = 0.6, [2] = 0, [3] = 0, [4] = 1}},
            ["Intervals"] = 0,
            ["Interval_Colour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
            ["Width"] = 500,
            ["Font"] = "BookAntiquaBold19",
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},      
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "PowerBar_Target" then
        return {
            ["Order"] = id,
            ["Type"] = "PowerBar_Target",
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Height"] = 20,
            ["Text"] = "<current>/<total> (<percentage>%)",
            ["Smooth"] = false,
            ["Alignment"] = "Middle",
            ["Colours"] = { [1] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
                            [2] = {[1] = 0.6, [2] = 0, [3] = 0, [4] = 1}},
            ["Intervals"] = 0,
            ["Interval_Colour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
            ["Width"] = 500,
            ["Font"] = "BookAntiquaBold19",
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},      
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "WrathBar" then
        return {
            ["Order"] = id,
            ["Type"] = "WrathBar",
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Height"] = 20,
            ["Smooth"] = true,
            ["Text"] = "<current>/<total> (<percentage>%)",
            ["Alignment"] = "Middle",
            ["Colours"] = { [1] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
                            [2] = {[1] = 0.6, [2] = 1, [3] = 0, [4] = 0}},
            ["Intervals"] = 0,
            ["Interval_Colour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0}, 
            ["Width"] = 500,
            ["Font"] = "BookAntiquaBold19",
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},        
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "ResourceBar" then
        return {
            ["Order"] = id,
            ["Type"] = "ResourceBar",
            ["Height"] = 10,
            ["Width"] = 320,
            ["Scale"] = 1,
            ["Full_Border"] = true,
            ["ShowText"] = true,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["ForegroundColour"] = {
                [1] = {[1] = 0.8, [2] = 0, [3] = 0.4,  [4] = 0},
                [2] = {[1] = 0.8, [2] = 0, [3] = 0.4,  [4] = 0},
                [3] = {[1] = 0.8, [2] = 0, [3] = 0.4,  [4] = 0},
                [4] = {[1] = 0.8, [2] = 0.4, [3] = 0.4,  [4] = 0},
                [5] = {[1] = 0.8, [2] = 0.4, [3] = 0.4,  [4] = 0},
                [6] = {[1] = 0.8, [2] = 0.4, [3] = 0.4,  [4] = 0},
                [7] = {[1] = 0.8, [2] = 0.4, [3] = 0,  [4] = 0},
                [8] = {[1] = 0.8, [2] = 0.4, [3] = 0,  [4] = 0},
                [9] = {[1] = 0.8, [2] = 0.4, [3] = 0,  [4] = 0},
            },
            ["BackgroundColour"] = {
                [1] = {[1] = 0.8, [2] = 0, [3] =0.8,  [4] = 0},
                [2] = {[1] = 0.8, [2] = 0, [3] =0.8,  [4] = 0},
                [3] = {[1] = 0.8, [2] = 0, [3] =0.8,  [4] = 0},
                [4] = {[1] = 0.8, [2] =0.8, [3] =0.8,  [4] = 0},
                [5] = {[1] = 0.8, [2] =0.8, [3] =0.8,  [4] = 0},
                [6] = {[1] = 0.8, [2] =0.8, [3] =0.8,  [4] = 0},
                [7] = {[1] = 0.8, [2] =0.8, [3] = 0,  [4] = 0},
                [8] = {[1] = 0.8, [2] =0.8, [3] = 0,  [4] = 0},
                [9] = {[1] = 0.8, [2] =0.8, [3] = 0,  [4] = 0},
            },
            ["BoxOffset"] = 3,
            ["BorderSize"] = 0,
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},
            ["Alignment"] = "Middle",
            ["RKLayout"] = false,
            ["RK_ForegroundColour_Heal"] = {[1] = 0.8, [2] = 0, [3] = 0.4,  [4] = 0},
            ["RK_ForegroundColour_DPS"] = {[1] = 0.8, [2] = 0.4, [3] = 0,  [4] = 0},
            ["RK_BackgroundColour_Heal"] = {[1] = 0.8, [2] = 0, [3] = 0.8,  [4] = 0},
            ["RK_BackgroundColour_DPS"] = {[1] = 0.8, [2] = 0.8, [3] = 0,  [4] = 0},
            ["BoxBackgroundColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},        
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "ValueBar" then
        return {
            ["Order"] = id,
            ["Type"] = "ValueBar",
            ["Height"] = 20,
            ["Width"] = 320,
            ["Visibility_Threshold"] = 9999,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BackgroundColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},
            ["BorderSize"] = 0,
            ["Vertical"] = false,
            ["Alignment"] = "Middle",
            ["BarAlignment"] = "Middle",
            ["BorderColour"] = {[1] = 0, [2] = 0.08, [3] = 0.08,  [4] = 0.08},
            ["ForegroundColour"] = {[1] = 1, [2] = 152/288, [3] = 0,  [4] = 0},
            ["EffectAdded"] = {},
            ["EffectRemoved"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectAdded_Target"] = {},    
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["Skills"] = {

            },     
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "BuffBar" then
        return {
            ["Order"] = id,
            ["Type"] = "BuffBar",
            ["Height"] = 20,
            ["Width"] = 100,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Alignment"] = "Middle",
            ["TextAlignment"] = "BottomCenter",
            ["StackTextAlignment"] = "TopCenter",
            ["Font_Timer"] = "FixedSys15",
            ["Font_Stack"] = "FixedSys15",
            ["Font_Stack_Colour"] = {[1] = 1, [2] = 1, [3] = 1,  [4] = 1},
            ["Font_Timer_Colour"] = {[1] = 1, [2] = 1, [3] = 1,  [4] = 1},
            ["Effects"] = {
            },
            ["Skills"] = {
            },
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["AllEffects"] = false,
            ["OnlyDebuffs"] = false,
            ["OnlyBuffs"] = false,
            ["AllHighlight"] = false,      
            ["X"] = 0,
            ["Y"] = 0,
            ["Offset"] = 0,
        }
    elseif type == "BuffBar_Target" then
        return {
            ["Order"] = id,
            ["Type"] = "BuffBar_Target",
            ["Height"] = 20,
            ["Width"] = 100,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Alignment"] = "Middle",
            ["TextAlignment"] = "BottomCenter",
            ["StackTextAlignment"] = "TopCenter",
            ["Font_Timer"] = "FixedSys15",
            ["Font_Stack"] = "FixedSys15",
            ["Font_Stack_Colour"] = {[1] = 1, [2] = 1, [3] = 1,  [4] = 1},
            ["Font_Timer_Colour"] = {[1] = 1, [2] = 1, [3] = 1,  [4] = 1},
            ["AllEffects"] = false,
            ["OnlyDebuffs"] = false,
            ["OnlyBuffs"] = false,
            ["AllHighlight"] = false,      
            ["Reset_On_Retarget"] = true,
            ["EffectAdded_Target"] = {}, 
            ["X"] = 0,
            ["Y"] = 0,
            ["Offset"] = 0,
        }
    elseif type == "Timer" then
        return {
            ["Order"] = id,
            ["Type"] = "Timer",
            ["Height"] = 40,
            ["Width"] = 300,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BackgroundColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},
            ["BorderSize"] = 0,
            ["Alignment"] = "Left",
            ["Font"] = "TrajanProBold16",
            ["Duration_Font"] = "FixedSys15",
            ["Icon"] = 1090551617,
            ["Text"] = "",
            ["BorderColour"] = {[1] = 0, [2] = 0.08, [3] = 0.08,  [4] = 0.08},
            ["BackgroundColour"] = {[1] = 0.9, [2] = 0, [3] = 0,  [4] = 0},
            ["ForegroundColour"] = {[1] = 1, [2] = 0.5, [3] = 0.5,  [4] = 0.5},
            ["AlwaysVisible"] = true,
            ["AllHighlight"] = false,
            ["EffectAdded"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectAdded_Target"] = {},    
            ["EffectRemoved"] = {},
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["Skills"] = {},
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},       
            ["X"] = 0,
            ["Y"] = 0,
            ["Unique"] = false,
        }
    elseif type == "TimerGroup" then
        return {
            ["Order"] = id,
            ["Type"] = "TimerGroup",
            ["Height"] = 40,
            ["Width"] = 300,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BackgroundColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},
            ["BorderSize"] = 0,
            ["Alignment"] = "Left",
            ["Icon"] = 1090551617,
            ["Text"] = "",
            ["BorderColour"] = {[1] = 0, [2] = 0.08, [3] = 0.08,  [4] = 0.08},
            ["Timer_BackgroundColour"] = {[1] = 0.6, [2] = 0, [3] = 0,  [4] = 0},
            ["Timer_ForegroundColour"] = {[1] = 1, [2] = 0.5, [3] = 0.5,  [4] = 0.5},
            ["AlwaysVisible"] = true,
            ["AllHighlight"] = false,
            ["EffectAdded"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectRemoved"] = {},
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["Skills"] = {},
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},        
            ["EffectAdded_Target"] = {},            
            ["X"] = 0,
            ["Y"] = 0,
            ["Timer_Height"] = 20,
            ["Timer_Font"] = "TrajanProBold16",
            ["Duration_Font"] = "FixedSys15",
            ["Static_Timers"] = false,
            ["Timer_Offset"] = 3,
            ["Timer_Sorting"] = "High to Low (Remaining duration)",
            ["Timer_Stack_Direction"] = "Down",
            ["Target_HealthThreshold"] = {},
            ["Reset_On_Retarget"] = false,
        }
    elseif type == "Text" then
        return {
            ["Order"] = id,
            ["Type"] = "Text",
            ["Height"] = 32,
            ["Width"] = 300,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BackgroundColour"] = {[1] = 0, [2] = 0, [3] = 0,  [4] = 0},
            ["BorderSize"] = 0,
            ["Alignment"] = "Middle",
            ["Font"] = "TrajanProBold16",
            ["Font_Colour"] = {[1] = 1, [2] = 1, [3] = 1,  [4] = 1},
            ["Text"] = "Text",
            ["BorderColour"] = {[1] = 0, [2] = 0.08, [3] = 0.08,  [4] = 0.08},
            ["AlwaysVisible"] = false,
            ["Effects"] = {
            },
            ["Conjuction"] = true,
            ["HealthThreshold"] = {},
            ["Instance_Only"] = false,            
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "Countdown" then
        return {
            ["Order"] = id,
            ["Type"] = "Countdown",
            ["Visibility_Threshold"] = 9999,
            ["EffectAdded"] = {},
            ["EffectRemoved"] = {},
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["Skills"] = {},
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectAdded_Target"] = {}, 
            ["Target_HealthThreshold"] = {},
        }
    elseif type == "Sound" then
        return {
            ["Order"] = id,
            ["Type"] = "Sound",
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},
            ["Skills"] = {},
            ["EffectAdded"] = {},
            ["EffectRemoved"] = {},
            ["HealthThreshold"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectAdded_Target"] = {}, 
            ["Target_HealthThreshold"] = {},
        }
    elseif type == "SkillLabel" then
        return {
            ["Order"] = id,
            ["Type"] = "SkillLabel",
            ["EnemyHits"] = {},
            ["PlayerHits"] = {},
            ["SayChat"] = {},
            ["PartyChat"] = {},
            ["StandardChat"] = {},
            ["Highlight_On_Effect"] = {},
            ["Skills"] = {},
            ["EffectAdded"] = {},
            ["EffectRemoved"] = {},
            ["EffectAdded_Group"] = {},
            ["EffectAdded_Target"] = {}, 
            ["Height"] = 32,
            ["Width"] = 32,
            ["Scale"] = 1,
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["BorderSize"] = 0,
            ["Alignment"] = "Middle",
            ["TextAlignment"] = "MiddleCenter",
            ["Font"] = "TrajanProBold16",
            ["Font_Colour"] = {[1] = 1, [2] = 1, [3] = 1, [4] = 1},
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},
            ["Icon"] = 1090551617,
            ["Visibility_Threshold"] = 9999,     
            ["Opacity_While_Active"] = 1,
            ["Opacity_While_Not_Active"] = 1,
            ["AlwaysVisible"] = false,
            ["Clockwise"] = true,
            ["Visual_Timer"] = true,
            ["Highlight"] = false,
            ["Hide_While_Not_Active"] = false,
            ["Hide_While_Active"] = false,
            ["HighlightWhileOff"] = false,
            ["X"] = 0,
            ["Y"] = 0,
        }
    elseif type == "Skills" then
        return {
            ["Order"] = id,
            ["Type"] = "Skills",
            ["OffsetAbove"] = 0,
            ["OffsetBelow"] = 0,
            ["Scale"] = 1,
            ["Skills"] = {
            },
            ["X"] = 0,
            ["Y"] = 0,
            ["Height"] = 32,
            ["Width"] = 32,
            ["Font_Colour"] = {[1] = 1, [2] = 1, [3] = 1, [4] = 1},
            ["BorderColour"] = {[1] = 0, [2] = 0, [3] = 0, [4] = 0},
            ["BorderSize"] = 0,
            ["Offset"] = 1,
        }
    end
end

function PrimeUIOptions:UITreeView(data1, width, valuebarNodes, name)

    for index, data in spairs(data1, function(t,a,b) return t[a]["Order"] < t[b]["Order"] end) do

        local removalFunction = function()
            self.currentUI["PrimeUI"][index] = nil

            local index = 1
            for key, value in spairs(self.currentUI["PrimeUI"], function(t,a,b) return t[a]["Order"] < t[b]["Order"] end) do
                self.currentUI["PrimeUI"][key]["Order"] = index
                index = index + 1
            end

            self.boxes[self.currentUI["Name"]]:SetParent(nil)
            self.boxes[self.currentUI["Name"]] = nil
            local count = tablelength(self.currentUI["PrimeUI"]) + 1
            local newTable = self:GetUIElementData(name, count)
            table.insert(self.currentUI["PrimeUI"], newTable)
            PrimeUIHandlerObject:Refresh(self.currentUI["Name"], self.currentUI["PrimeUI"])
            self.data = PrimeUIHandlerObject:GetData()
            for _, data in pairs(self.data) do
                if not(self.boxes[data["Name"]]) then
                    self.boxes[data["Name"]] = self:CreateUIItem(self.SetupContainer, data)
                end
            end
            self.boxes[self.currentUI["Name"]]:SetVisible(true)
        end

        local movefunction = function(up)
            local newOrder = data["Order"]
            local oldOrder = data["Order"]

            if up then
                if newOrder == 1 then return end
                newOrder = newOrder - 1
            else
                if newOrder + 1 > tablelength(self.currentUI["PrimeUI"]) then
                    return 
                end
                newOrder = newOrder + 1
            end

            data["Order"] = -1

            for key, value in spairs(self.currentUI["PrimeUI"], function(t,a,b) return t[a]["Order"] < t[b]["Order"] end) do
                if self.currentUI["PrimeUI"][key]["Order"] == newOrder then
                    self.currentUI["PrimeUI"][key]["Order"] = oldOrder
                end
            end

            for key, value in spairs(self.currentUI["PrimeUI"], function(t,a,b) return t[a]["Order"] < t[b]["Order"] end) do
                if self.currentUI["PrimeUI"][key]["Order"] == -1 then
                    self.currentUI["PrimeUI"][key]["Order"] = newOrder
                end
            end

            self.boxes[self.currentUI["Name"]]:SetParent(nil)
            self.boxes[self.currentUI["Name"]] = nil
            local count = tablelength(self.currentUI["PrimeUI"]) + 1
            local newTable = self:GetUIElementData(name, count)
            table.insert(self.currentUI["PrimeUI"], newTable)
            PrimeUIHandlerObject:Refresh(self.currentUI["Name"], self.currentUI["PrimeUI"])
            self.data = PrimeUIHandlerObject:GetData()
            for _, data in pairs(self.data) do
                if not(self.boxes[data["Name"]]) then
                    self.boxes[data["Name"]] = self:CreateUIItem(self.SetupContainer, data)
                end
            end
            self.boxes[self.currentUI["Name"]]:SetVisible(true)
        end

        --valuebar:AddItem(self:Border(width))
        local typeHeader = self:Label(width, data["Type"], removalFunction, movefunction) 
        valuebarNodes:Add(typeHeader)
        local UINodes = typeHeader:GetChildNodes()
        --UINodes:Add(self:Border(width))

        local optionHeader = ""
        local subUINodes = nil
        
        for key, value in spairs(data, function(t,a,b) 
            if self:SortOrder(a, t[a]) == self:SortOrder(b, t[b]) then
                return a < b
            else
                return self:SortOrder(a, t[a]) < self:SortOrder(b, t[b])
            end
        end) do
            local newOptionHeader = self.SortOrderTable[self:SortOrder(key, value)]
            if newOptionHeader and newOptionHeader ~= optionHeader then
                optionHeader = newOptionHeader
                local subLabel = self:OptionLabel(width, tostring(optionHeader))
                subUINodes = subLabel:GetChildNodes()
                UINodes:Add(subLabel)
                --UINodes:Add(self:Border(width))
            end    
            local skip = false
            if key == "Order" or key == "Type" then
                skip = true
            elseif key == "Icon" then
                local element = self:IconInput(width, key, value, (function(newIcon)
                    data[key] = newIcon
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            elseif  key == "Effects" or key == "Skills" or key == "PlayerHits" or key == "EnemyHits" or 
                    key == "EffectAdded" or key == "EffectRemoved" or key == "PartyChat" or key == "StandardChat" or 
                    key == "SayChat" or key == "HealthThreshold" or key == "EffectAdded_Group" or key == "EffectAdded_Target" or
                    key == "Target_HealthThreshold" or key == "Highlight_On_Effect" then
                local element = self:ListBoxInput(width, key, value, data["Type"], (function(effectName, newState)
                    if newState and data[key][effectName] then
                        ChatOutput("Failed to add item: Item already exists")
                        return true
                    end
                    data[key][effectName] = newState
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            elseif string.find(key, "Colour") then
                local element = self:ColourInput(width, key, value, (function(newColour, index)
                    if index then
                        data[key][index] = newColour
                    else
                        data[key] = newColour
                    end
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                self:InfoButton(element, width, self.optionInfoText["Colour"])
                subUINodes:Add(element)
            elseif string.find(key, "Font") then
                local element = self:FontInput(width, key, value, (function(newFont)
                    data[key] = newFont
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText["Font"])
                subUINodes:Add(element)
            elseif string.find(key, "Timer_Stack_Direction") then
                local element = self:TimerStackAlignmentInput(width, key, value, (function(alignment)
                    data[key] = alignment
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            elseif string.find(key, "Timer_Sorting") then
                local element = self:TimerGroupAlignmentInput(width, key, value, (function(alignment)
                    data[key] = alignment
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, text)
                subUINodes:Add(element)
            elseif string.find(key, "TextAlignment") then
                local element = self:TextAlignmentInput(width, key, value, (function(alignment)
                    data[key] = alignment
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText["TextAlignment"])
                subUINodes:Add(element)
            elseif string.find(key, "Alignment") then
                local element = self:AlignmentInput(width, key, value, (function(alignment)
                    data[key] = alignment
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText["Alignment"])
                subUINodes:Add(element)
            elseif isnumber(value) then
                local element = self:TextInput(width, key, value, (function(text) 
                    local number = tonumber(text)
                    if number then
                        data[key] = number
                        
                        PrimeUIHandlerObject:Refresh(name, data1)
                    end
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            elseif value == true or value == false then
                local element = self:BoolInput(width, key, value, (function(bool) 
                    data[key] = bool
                    
                    PrimeUIHandlerObject:Refresh(name, data1)
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            elseif key == "Text" then
                local element = self:BigTextInput(width, key, value, (function(text) 
                    if text then
                        data[key] = text
                        
                        PrimeUIHandlerObject:Refresh(name, data1)
                    end
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            elseif type(value) == "string" then
                local element = self:TextInput(width, key, value, (function(text) 
                    if text then
                        data[key] = text
                        
                        PrimeUIHandlerObject:Refresh(name, data1)
                    end
                end))
                subUINodes:Add(element)
                self:InfoButton(element, width, self.optionInfoText[key])
                subUINodes:Add(element)
            else
                ChatOutput("Skipped")
                ChatOutput(key)
            end

            if not(skip) then subUINodes:Add(self:Border(width)) end
        end
    end
end

function PrimeUIOptions:InfoHover(type, text, width)
    
end

function PrimeUIOptions:TextInput(width, text, currentValue, func)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 50)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
	valuebar.searchBar:SetParent(valuebar)
	valuebar.searchBar:SetSize(300, 20)
	valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText(currentValue)

    valuebar.searchBar.TextChanged = function()
        func(tostring(valuebar.searchBar:GetText()))
    end

    return valuebar
end

function PrimeUIOptions:SortOrder(key, value)
    if key == "Order" or key == "Type" then
        return 9999
    elseif key == "Icon" then
        return 35
    elseif key == "Effects" or key == "Skills" or key == "PlayerHits" or key == "EnemyHits" or key == "EffectAdded" or key == "EffectRemoved" or key == "PartyChat" or key == "StandardChat" or key == "SayChat" or key == "HealthThreshold" or key == "EffectAdded_Group" or key == "EffectAdded_Target" then
        return 50
    elseif string.find(key, "Colour") then
        return 40
    elseif string.find(key, "Timer_Sorting") then
        return 22 
    elseif string.find(key, "Font") then
        return 24
    elseif string.find(key, "Timer_Stack_Direction") then
        return 23
    elseif string.find(key, "TextAlignment") then
        return 21
    elseif string.find(key, "Alignment") then
        return 20
    elseif isnumber(value) then
        return 10
    elseif value == true or value == false then
        return 30
    elseif key == "Text" then
        return 1
    elseif type(value) == "string" then
        return 2
    else
        return 9999
    end
end

function PrimeUIOptions:BigTextInput(width, text, currentValue, func)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 120)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
	valuebar.searchBar:SetParent(valuebar)
	valuebar.searchBar:SetSize(300, 80)
	valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText(currentValue)
    valuebar.searchBar:SetMultiline(true)

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(80, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(20, 60)
    valuebar.Button:SetText("Change")

    valuebar.Button.Click = function()
        func(tostring(valuebar.searchBar:GetText()))
    end

    return valuebar
end

function PrimeUIOptions:BoolInput(width, text, currentValue, func)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 50)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.checkBox = Turbine.UI.Lotro.CheckBox()
	valuebar.checkBox:SetParent(valuebar)
	valuebar.checkBox:SetSize(width - 150, 20)
	valuebar.checkBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.checkBox:SetPosition(150, 20)
    valuebar.checkBox:SetText(text)
    valuebar.checkBox:SetChecked(currentValue)

    valuebar.checkBox.CheckedChanged = function()
        func(valuebar.checkBox:IsChecked())
    end

    return valuebar
end

function PrimeUIOptions:Border(width)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 9)
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    --valuebar:SetBackColor(Turbine.UI.Color(0.6, 0.125, 0.125, 0.125))
    valuebar:SetBackground(0x4110CD4D)

    --[[
    valuebar.topBar = Turbine.UI.Control()
    valuebar.topBar:SetParent(valuebar)
    valuebar.topBar:SetSize(width, 3)
    valuebar.topBar:SetPosition(0, 0)
    valuebar.topBar:SetMouseVisible(false)
    valuebar.topBar:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.topBar:SetBackColor(Turbine.UI.Color(0.6, 1, 1, 1))

    valuebar.lowBar = Turbine.UI.Control()
    valuebar.lowBar:SetParent(valuebar)
    valuebar.lowBar:SetSize(width, 3)
    valuebar.lowBar:SetPosition(0, 6)
    valuebar.lowBar:SetMouseVisible(false)
    valuebar.lowBar:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.lowBar:SetBackColor(Turbine.UI.Color(0.6, 1, 1, 1))
    --]]

    

    return valuebar
end

function PrimeUIOptions:ListBoxBorder(width)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 9)
    valuebar:SetMouseVisible(false)
    valuebar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar:SetBackground(0x4110CD4D)

    return valuebar
end

function PrimeUIOptions:Label(width, text, func1, func2)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    --valuebar:SetBackColor(Turbine.UI.Color(0.6, 0.125, 0.125, 0.125))
    valuebar:SetSize(width, 65)

    valuebar.left = Turbine.UI.Control()
    valuebar.left:SetParent(valuebar)
    valuebar.left:SetSize(80,80)
    valuebar.left:SetBackground(0x411C0E92)
    valuebar.left:SetPosition(0, -5)
    valuebar.left:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    valuebar.middle = Turbine.UI.Control()
    valuebar.middle:SetParent(valuebar)
    valuebar.middle:SetSize(width - 160,80)
    valuebar.middle:SetBackground(0x411C0E94)
    valuebar.middle:SetPosition(80, -5)
    valuebar.middle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    valuebar.right = Turbine.UI.Control()
    valuebar.right:SetParent(valuebar)
    valuebar.right:SetSize(80,80)
    valuebar.right:SetBackground(0x411C0E9D)
    valuebar.right:SetPosition(width - 80, -5)
    valuebar.right:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
 
    valuebar.label = Turbine.UI.Label()
    valuebar.label:SetParent(valuebar)
    valuebar.label:SetSize(width, 70)
    valuebar.label:SetPosition(30, 0)
    valuebar.label:SetMouseVisible(false)
    valuebar.label:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    --valuebar.label:SetBackColor(Turbine.UI.Color(0.6, 0.125, 0.125, 0.125))
    valuebar.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    valuebar.label:SetFont(Turbine.UI.Lotro.Font.TrajanProBold24)
    valuebar.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    valuebar.label:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    valuebar.label:SetText(text)

    valuebar.upButton = Turbine.UI.Lotro.Button()
	valuebar.upButton:SetParent(valuebar)
	valuebar.upButton:SetSize(200, 40)
	valuebar.upButton:SetPosition(width - 320, 15)
    valuebar.upButton:SetText("Move Draw Order Up")

    valuebar.upButton.Click = function()
        func2(true)
    end

    valuebar.downButton = Turbine.UI.Lotro.Button()
	valuebar.downButton:SetParent(valuebar)
	valuebar.downButton:SetSize(200, 40)
	valuebar.downButton:SetPosition(width - 320, 35)
    valuebar.downButton:SetText("Move Draw Order Down")

    valuebar.downButton.Click = function()
        func2(false)
    end

    valuebar.removeButton = Turbine.UI.Lotro.Button()
	valuebar.removeButton:SetParent(valuebar)
	valuebar.removeButton:SetSize(60, 40)
	valuebar.removeButton:SetPosition(width - 120, 25)
    valuebar.removeButton:SetText("Delete")

    valuebar.removeButton.Click = function()
        func1()
    end

    return valuebar
end

function PrimeUIOptions:OptionLabel(width, text)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 50)

    valuebar.border = Turbine.UI.Control()
    valuebar.border:SetParent(valuebar)
    valuebar.border:SetSize(width, 50)
    valuebar.border:SetMouseVisible(false)
    valuebar.border:SetBackColor(Turbine.UI.Color(0, 0, 0))

    valuebar.label = Turbine.UI.Label()
    valuebar.label:SetParent(valuebar)
    valuebar.label:SetSize(width - 4, 50 - 4)
    valuebar.label:SetPosition(2, 2)
    valuebar.label:SetMouseVisible(false)
    valuebar.label:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.label:SetBackColor(Turbine.UI.Color(0.8, 0.125, 0.125, 0.125))
    valuebar.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.label:SetFont(Turbine.UI.Lotro.Font.TrajanProBold25)
    valuebar.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    valuebar.label:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    valuebar.label:SetText(text)

    return valuebar
end

function PrimeUIOptions:TimerStackAlignmentInput(width, text, currentValue, func)

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 60)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(200, 20)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, value in pairs(self.timeStackAlignments) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(50, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(200, 20)
    valuebar.Button:SetText(currentValue)

        
    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            valuebar.Button:SetText(name)
            func(name)
        end
        
        menuItems:Add(menuItem)
    end

    valuebar.Button.Click = function()
        menu:ShowMenu()
    end

    return valuebar
end

function PrimeUIOptions:TimerGroupAlignmentInput(width, text, currentValue, func)

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 60)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, value in pairs(self.timerGroupAlignments) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(300, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(150, 20)
    valuebar.Button:SetText(currentValue)

        
    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            valuebar.Button:SetText(name)
            func(name)
        end
        
        menuItems:Add(menuItem)
    end

    valuebar.Button.Click = function()
        menu:ShowMenu()
    end

    return valuebar
end

function PrimeUIOptions:TextAlignmentInput(width, text, currentValue, func)

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 60)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, value in pairs(self.textAlignments) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(150, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(150, 20)
    valuebar.Button:SetText(currentValue)

        
    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            valuebar.Button:SetText(name)
            func(name)
        end
        
        menuItems:Add(menuItem)
    end

    valuebar.Button.Click = function()
        menu:ShowMenu()
    end

    return valuebar
end

function PrimeUIOptions:AlignmentInput(width, text, currentValue, func)

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 60)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, value in pairs(self.alignments) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(150, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(150, 20)
    valuebar.Button:SetText(currentValue)

        
    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            valuebar.Button:SetText(name)
            func(name)
        end
        
        menuItems:Add(menuItem)
    end

    valuebar.Button.Click = function()
        menu:ShowMenu()
    end

    return valuebar
end

function PrimeUIOptions:FontInput(width, text, currentValue, func)

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 60)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
    --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    local menu = Turbine.UI.ContextMenu()
    local menuItems = menu:GetItems()

    local sortedKeys = {}
    for key, value in pairs(Turbine.UI.Lotro.Font) do table.insert( sortedKeys, key ) end
    table.sort( sortedKeys )

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(150, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(150, 20)
    valuebar.Button:SetText(currentValue)

        
    for key, name in pairs(sortedKeys) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            valuebar.Button:SetText(name)
            func(name)
        end
        
        menuItems:Add(menuItem)
    end

    valuebar.Button.Click = function()
        menu:ShowMenu()
    end

    return valuebar
end

function PrimeUIOptions:ColourInput(width, text, currentValue, func)

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 160)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
   --valuebar:SetBackColor(GetRandomColour())

    if currentValue then
        if type(currentValue[1]) == "table" then
            for key, value in pairs(currentValue) do
                self:SingleColourOptions(valuebar, width, text, value, func, key)
                valuebar:SetSize(width, 160 * key)
            end
            return valuebar
        else
            self:SingleColourOptions(valuebar, width, text, currentValue, func)
            return valuebar
        end
    end

    return valuebar
end

function PrimeUIOptions:SingleColourOptions(parent, width, text, currentValue, func, index)
    local y = 0
    if index then
        y = (index - 1) * 160
    end
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetParent(parent)
    valuebar:SetSize(width, 160)
    valuebar:SetPosition(0, y)
    valuebar:SetMouseVisible(false)
   --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    valuebar.currentLabel = Turbine.UI.Label()
	valuebar.currentLabel:SetParent(valuebar)
	valuebar.currentLabel:SetSize(120, 20)
	valuebar.currentLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.currentLabel:SetPosition(0, 130)
    valuebar.currentLabel:SetText("Current Colour:")

    valuebar.Current = Turbine.UI.Control()
    valuebar.Current:SetParent(valuebar)
    valuebar.Current:SetSize(32, 32)
    valuebar.Current:SetMouseVisible(false)
    valuebar.Current:SetPosition(115, 126)
    valuebar.Current:SetBackColor(Turbine.UI.Color(currentValue[1], currentValue[2], currentValue[3], currentValue[4]))

    valuebar.newLabel = Turbine.UI.Label()
	valuebar.newLabel:SetParent(valuebar)
	valuebar.newLabel:SetSize(100, 20)
	valuebar.newLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.newLabel:SetPosition(155, 130)
    valuebar.newLabel:SetText("New Colour:")

    valuebar.Example = Turbine.UI.Control()
    valuebar.Example:SetParent(valuebar)
    valuebar.Example:SetSize(32, 32)
    valuebar.Example:SetMouseVisible(false)
    valuebar.Example:SetPosition(250, 126)
    --valuebar.Example:SetBackColor(GetRandomColour())

    -- 1
    valuebar.scrollbar1 = Turbine.UI.Lotro.ScrollBar()
	valuebar.scrollbar1:SetParent(valuebar)
    valuebar.scrollbar1:SetValue(currentValue[1] * 100)
	valuebar.scrollbar1:SetSize(250, 10)
    valuebar.scrollbar1:SetOrientation(Turbine.UI.Orientation.Horizontal)
	valuebar.scrollbar1:SetPosition(150, 20)

    valuebar.scrollbar1label = Turbine.UI.Label()
	valuebar.scrollbar1label:SetParent(valuebar)
	valuebar.scrollbar1label:SetSize(150, 20)
	valuebar.scrollbar1label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.scrollbar1label:SetPosition(400, 15)
    valuebar.scrollbar1label:SetText("A: " .. tostring(currentValue[1]))

    -- 2
    valuebar.scrollbar2 = Turbine.UI.Lotro.ScrollBar()
	valuebar.scrollbar2:SetParent(valuebar)
	valuebar.scrollbar2:SetSize(250, 10)
    valuebar.scrollbar2:SetOrientation(Turbine.UI.Orientation.Horizontal)
	valuebar.scrollbar2:SetPosition(150, 40)
    valuebar.scrollbar2:SetValue(currentValue[2] * 100)

    valuebar.scrollbar2label = Turbine.UI.Label()
	valuebar.scrollbar2label:SetParent(valuebar)
	valuebar.scrollbar2label:SetSize(150, 20)
	valuebar.scrollbar2label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.scrollbar2label:SetPosition(400, 35)
    valuebar.scrollbar2label:SetText("R: " .. tostring(currentValue[2]))
    
    -- 3
    valuebar.scrollbar3 = Turbine.UI.Lotro.ScrollBar()
	valuebar.scrollbar3:SetParent(valuebar)
	valuebar.scrollbar3:SetSize(250, 10)
    valuebar.scrollbar3:SetOrientation(Turbine.UI.Orientation.Horizontal)
	valuebar.scrollbar3:SetPosition(150, 60)
    valuebar.scrollbar3:SetValue(currentValue[3] * 100)

    valuebar.scrollbar3label = Turbine.UI.Label()
	valuebar.scrollbar3label:SetParent(valuebar)
	valuebar.scrollbar3label:SetSize(150, 20)
	valuebar.scrollbar3label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.scrollbar3label:SetPosition(400, 55)
    valuebar.scrollbar3label:SetText("G: " .. tostring(currentValue[3]))

    -- 4
    valuebar.scrollbar4 = Turbine.UI.Lotro.ScrollBar()
    valuebar.scrollbar4:SetParent(valuebar)
    valuebar.scrollbar4:SetSize(250, 10)
    valuebar.scrollbar4:SetOrientation(Turbine.UI.Orientation.Horizontal)
    valuebar.scrollbar4:SetPosition(150, 80)
    valuebar.scrollbar4:SetValue(currentValue[4] * 100)

    valuebar.scrollbar4label = Turbine.UI.Label()
    valuebar.scrollbar4label:SetParent(valuebar)
    valuebar.scrollbar4label:SetSize(150, 20)
    valuebar.scrollbar4label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.scrollbar4label:SetPosition(400, 75)
    valuebar.scrollbar4label:SetText("B: " .. tostring(currentValue[4]))

    valuebar.scrollbar1.ValueChanged = function()
        valuebar.scrollbar1label:SetText("A:  " .. valuebar.scrollbar1:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
    end

    valuebar.scrollbar2.ValueChanged = function()
        valuebar.scrollbar2label:SetText("R:  " .. valuebar.scrollbar2:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
    end

    valuebar.scrollbar3.ValueChanged = function()
        valuebar.scrollbar3label:SetText("G:  " .. valuebar.scrollbar3:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
    end


    valuebar.scrollbar4.ValueChanged = function()
        valuebar.scrollbar4label:SetText("B:  " .. valuebar.scrollbar4:GetValue() / 100)
        valuebar.Example:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
    end

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(150, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(300, 130)
    valuebar.Button:SetText("Change colour")

    valuebar.Button.Click = function()
        valuebar.Current:SetBackColor(Turbine.UI.Color(valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100))
        func({valuebar.scrollbar1:GetValue() / 100, valuebar.scrollbar2:GetValue() / 100, valuebar.scrollbar3:GetValue() / 100, valuebar.scrollbar4:GetValue() / 100}, index)
    end
end

function PrimeUIOptions:IconInput(width, text, currentValue, func)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 60)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
   --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    valuebar.EffectTimerIconBox = Turbine.UI.Control()
    valuebar.EffectTimerIconBox:SetParent(valuebar)
    valuebar.EffectTimerIconBox:SetPosition(150, 15)
    valuebar.EffectTimerIconBox:SetSize(32,32)
    valuebar.EffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.EffectTimerIconBox:SetBackground(tonumber(currentValue) or currentValue)
    valuebar.EffectTimerIconBox:SetVisible(true)

    valuebar.TimerIconTextBox = Turbine.UI.Lotro.TextBox()
    valuebar.TimerIconTextBox:SetParent(valuebar)
    valuebar.TimerIconTextBox:SetSize(110, 30)
    valuebar.TimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.TimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.TimerIconTextBox:SetMouseVisible(false)
    valuebar.TimerIconTextBox:SetPosition(190, 15)
    valuebar.TimerIconTextBox:SetMultiline(false)
    valuebar.TimerIconTextBox:SetText(currentValue)

    valuebar.EffectTimerIconBox.MouseClick = function()
        if not(self.iconWindow) then self.iconWindow = IconOverview() end
        if self.iconWindow then 
            self.iconWindow:SetVisible(true) 
            self.iconWindow:Register(valuebar.EffectTimerIconBox, valuebar.TimerIconTextBox)
        end
    end

    valuebar.EffectTimerIconBox.MouseDoubleClick = function()
        valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
        valuebar.TimerIconTextBox:SetText(0x41007F41)
    end

    valuebar.Button = Turbine.UI.Lotro.Button()
    valuebar.Button:SetParent(valuebar)
    valuebar.Button:SetSize(150, 100)
    valuebar.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.Button:SetMouseVisible(false)
    valuebar.Button:SetPosition(300, 20)
    valuebar.Button:SetText("Change")

    valuebar.Button.Click = function()
       func(tostring(valuebar.TimerIconTextBox:GetText()))
    end

    return valuebar
end

function PrimeUIOptions:ListBoxInput(width, text, currentValue, type, func)
    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetSize(width, 200)
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
   --valuebar:SetBackColor(GetRandomColour())

    valuebar.label = Turbine.UI.Label()
	valuebar.label:SetParent(valuebar)
	valuebar.label:SetSize(150, 40)
	valuebar.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	valuebar.label:SetPosition(0, 20)
    valuebar.label:SetText(text)

    valuebar.listbox = Turbine.UI.ListBox()
    valuebar.listbox:SetParent(valuebar)
    valuebar.listbox:SetSize(width - 10, 100)
    valuebar.listbox:SetPosition(0, 100)
    valuebar.listbox:SetVisible(tablelength(currentValue) > 0)

    valuebar.Scrollbar = Turbine.UI.Lotro.ScrollBar();
    valuebar.Scrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    valuebar.Scrollbar:SetParent(valuebar)
    valuebar.Scrollbar:SetPosition(width - 10, 100)
    valuebar.Scrollbar:SetSize(10, 100)
    valuebar.listbox:SetVerticalScrollBar(valuebar.Scrollbar)

    if type == "SkillLabel" and text == "Highlight_On_Effect" then
        self:HighlightInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    elseif type == "Icon" or type == "Text" then
        self:BoolListboxInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    elseif type == "ValueBar" or type == "Countdown" or type == "TimerIcon" or type == "SkillLabel" then
        self:NumberInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    elseif type == "Sound" then
        self:SoundListboxInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    elseif type == "BuffBar" or type == "Timer" or type == "BuffBar_Target" then
        self:BuffBarInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    elseif type == "Skills" then
        self:SkillBarInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    elseif type == "TimerGroup" then
        self:TimerGroupInput(valuebar, valuebar.listbox, currentValue, width, func, text)
    end

    return valuebar
end

function PrimeUIOptions:TimerGroupInput(valuebar, listbox, currentValue, width, func, type)
    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 40)

        effectcontrol.effectlabelInfo = Turbine.UI.Label()
        effectcontrol.effectlabelInfo:SetParent(effectcontrol)
        effectcontrol.effectlabelInfo:SetSize(50, 20)
        effectcontrol.effectlabelInfo:SetPosition(180, 0)
        effectcontrol.effectlabelInfo:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabelInfo:SetText("Icon:")

        effectcontrol.iconBox = Turbine.UI.Label()
        effectcontrol.iconBox:SetParent(effectcontrol)
        effectcontrol.iconBox:SetSize(50, 20)
        effectcontrol.iconBox:SetPosition(180, 20)
        effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconBox:SetText("Time:")

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(250, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        effectcontrol.iconBox = Turbine.UI.Label()
        effectcontrol.iconBox:SetParent(effectcontrol)
        effectcontrol.iconBox:SetSize(150, 20)
        effectcontrol.iconBox:SetPosition(230, 0)
        effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconBox:SetText(tostring(value["Icon"]))

        effectcontrol.timerBox = Turbine.UI.Label()
        effectcontrol.timerBox:SetParent(effectcontrol)
        effectcontrol.timerBox:SetSize(150, 20)
        effectcontrol.timerBox:SetPosition(230, 20)
        effectcontrol.timerBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.timerBox:SetText(tostring(value["Time"]))

        effectcontrol.effectbox = Turbine.UI.Lotro.CheckBox()
        effectcontrol.effectbox:SetParent(effectcontrol)
        effectcontrol.effectbox:SetSize(100, 20)
        effectcontrol.effectbox:SetPosition(0, 20)
        effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectbox:SetText("Unique")
        effectcontrol.effectbox:SetChecked(value["Unique"])

        effectcontrol.iconMatchBox = Turbine.UI.Lotro.TextBox()
        effectcontrol.iconMatchBox:SetParent(effectcontrol)
        effectcontrol.iconMatchBox:SetSize(100, 20)
        effectcontrol.iconMatchBox:SetPosition(80, 20)
        effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconMatchBox:SetText(value["Message"])

        if type == "Target_HealthThreshold" then
            effectcontrol.iconMatchBox.TextChanged = function()
                func(key, nil)
                func(key, {["Icon"] =  value["Icon"], ["Time"] = value["Time"], ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText()), ["Number"] = value["Number"], ["Operator"] = value["Operator"]})
            end

            effectcontrol.effectbox.CheckedChanged = function()
                func(key, nil)
                func(key, {["Icon"] = value["Icon"], ["Time"] = value["Time"], ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText()), ["Number"] = value["Number"], ["Operator"] = value["Operator"]})
            end
        else
            effectcontrol.iconMatchBox.TextChanged = function()
                func(key, nil)
                func(key, {["Icon"] =  value["Icon"], ["Time"] = value["Time"], ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText())})
            end

            effectcontrol.effectbox.CheckedChanged = function()
                func(key, nil)
                func(key, {["Icon"] = value["Icon"], ["Time"] = value["Time"], ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText())})
            end
        end

        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(200, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText("Name")

    valuebar.timeBar = Turbine.UI.Lotro.TextBox()
    valuebar.timeBar:SetParent(valuebar)
    valuebar.timeBar:SetSize(100, 20)
    valuebar.timeBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.timeBar:SetPosition(350, 20)
    valuebar.timeBar:SetText("Time")

    if type == "Target_HealthThreshold" then
        valuebar.targetNumberBar = Turbine.UI.Lotro.TextBox()
        valuebar.targetNumberBar:SetParent(valuebar)
        valuebar.targetNumberBar:SetSize(100, 20)
        valuebar.targetNumberBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.targetNumberBar:SetPosition(350, 40)
        valuebar.targetNumberBar:SetText("Threshold")
    end

    if type == "EffectAdded" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "EffectRemoved" or type == "Effects" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.effectWindow) then self.effectWindow = HistoricalEffectOverview() end
            if self.effectWindow then 
                self.effectWindow:SetVisible(true) 
                self.effectWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "Skills" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.skillWindow) then self.skillWindow = SkillOverview() end
            if self.skillWindow then 
                self.skillWindow:SetVisible(true) 
                self.skillWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "Target_HealthThreshold" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("<=")

        valuebar.browseMenu = Turbine.UI.ContextMenu()
        valuebar.browseMenuItems = valuebar.browseMenu :GetItems()

        local menuItem1 = Turbine.UI.MenuItem("<=")
        valuebar.browseMenuItems:Add(menuItem1)
        menuItem1.Click = function()
            valuebar.browse:SetText("<=")
        end

        local menuItem2 = Turbine.UI.MenuItem(">=")
        valuebar.browseMenuItems:Add(menuItem2)
        menuItem2.Click = function()
            valuebar.browse:SetText(">=")
        end
    
        valuebar.browse.Click = function()
            valuebar.browseMenu:ShowMenu()
        end
    end

    valuebar.EffectTimerIconBox = Turbine.UI.Control()
    valuebar.EffectTimerIconBox:SetParent(valuebar)
    valuebar.EffectTimerIconBox:SetPosition(150, 60)
    valuebar.EffectTimerIconBox:SetSize(32,32)
    valuebar.EffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
    valuebar.EffectTimerIconBox:SetVisible(true)

    valuebar.TimerIconTextBox = Turbine.UI.Lotro.TextBox()
    valuebar.TimerIconTextBox:SetParent(valuebar)
    valuebar.TimerIconTextBox:SetSize(110, 30)
    valuebar.TimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.TimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.TimerIconTextBox:SetMouseVisible(false)
    valuebar.TimerIconTextBox:SetPosition(190, 60)
    valuebar.TimerIconTextBox:SetMultiline(false)
    valuebar.TimerIconTextBox:SetText(0x41007F41)

    valuebar.EffectTimerIconBox.MouseClick = function()
        if not(self.iconWindow) then self.iconWindow = IconOverview() end
        if self.iconWindow then 
            self.iconWindow:SetVisible(true) 
            self.iconWindow:Register(valuebar.EffectTimerIconBox, valuebar.TimerIconTextBox)
        end
    end

    valuebar.EffectTimerIconBox.MouseDoubleClick = function()
        valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
        valuebar.TimerIconTextBox:SetText(0x41007F41)
    end

    valuebar.effectbox = Turbine.UI.Lotro.CheckBox()
    valuebar.effectbox:SetParent(valuebar)
    valuebar.effectbox:SetSize(100, 20)
    valuebar.effectbox:SetPosition(250, 0)
    valuebar.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.effectbox:SetText("Unique")

    valuebar.matchBox = Turbine.UI.Lotro.TextBox()
    valuebar.matchBox:SetParent(valuebar)
    valuebar.matchBox:SetSize(100, 20)
    valuebar.matchBox:SetPosition(300, 60)
    valuebar.matchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.matchBox:SetText("Text")

    if type == "EffectAdded" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "EffectRemoved" or type == "Effects" or type == "Skills" then
        valuebar.iconBox = Turbine.UI.Lotro.CheckBox()
        valuebar.iconBox:SetParent(valuebar)
        valuebar.iconBox:SetSize(200, 20)
        valuebar.iconBox:SetPosition(150, 40)
        valuebar.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        if type == "Skills" then
            valuebar.iconBox:SetText("Take icon from skill")
        else
            valuebar.iconBox:SetText("Take icon from effect")
        end

        valuebar.timerCheckBox = Turbine.UI.Lotro.CheckBox()
        valuebar.timerCheckBox:SetParent(valuebar)
        valuebar.timerCheckBox:SetSize(200, 20)
        valuebar.timerCheckBox:SetPosition(320, 40)
        valuebar.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        if type == "Skills" then
            valuebar.timerCheckBox:SetText("Take time from skill")
        else
            valuebar.timerCheckBox:SetText("Take time from effect")
        end
    end

    valuebar.addEffect.Click = function()
        local time = tonumber(tostring((valuebar.timeBar:GetText())))
        if valuebar.timerCheckBox and valuebar.timerCheckBox:IsChecked() then
            time = true
        end
        if not(time) then ChatOutput("Not all fields are correctly filled!"); return end
        local newEffect = tostring(valuebar.searchBar:GetText())
        local icon = tostring(valuebar.TimerIconTextBox:GetText())
        if valuebar.iconBox and valuebar.iconBox:IsChecked() then
            icon = true
        end
        local thesholdOperator = ""
        local thesholdNumber = 0
        if type == "Target_HealthThreshold" then
            thesholdOperator = tonumber(tostring(valuebar.browse:GetText()))
            thesholdNumber = tonumber(tostring(valuebar.targetNumberBar:GetText()))
        end
        local highlight = valuebar.effectbox:IsChecked()

        local message = tostring(valuebar.matchBox:GetText())
        if type == "Target_HealthThreshold" then
            if func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Unique"] = highlight, ["Message"] = message, ["Number"] = tonumber(tostring(valuebar.targetNumberBar:GetText())), ["Operator"] = tostring(valuebar.browse:GetText())}) then
                return
            end
        else
            if func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Unique"] = highlight, ["Message"] = message}) then
                return
            end
        end
        if newEffect then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 40)

            
            effectcontrol.effectlabelInfo = Turbine.UI.Label()
            effectcontrol.effectlabelInfo:SetParent(effectcontrol)
            effectcontrol.effectlabelInfo:SetSize(50, 20)
            effectcontrol.effectlabelInfo:SetPosition(180, 0)
            effectcontrol.effectlabelInfo:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabelInfo:SetText("Icon:")

            effectcontrol.iconBox = Turbine.UI.Label()
            effectcontrol.iconBox:SetParent(effectcontrol)
            effectcontrol.iconBox:SetSize(50, 20)
            effectcontrol.iconBox:SetPosition(180, 20)
            effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconBox:SetText("Time:")

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(250, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            effectcontrol.iconBox = Turbine.UI.Label()
            effectcontrol.iconBox:SetParent(effectcontrol)
            effectcontrol.iconBox:SetSize(150, 20)
            effectcontrol.iconBox:SetPosition(230, 0)
            effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconBox:SetText(tostring(icon))

            effectcontrol.timerBox = Turbine.UI.Label()
            effectcontrol.timerBox:SetParent(effectcontrol)
            effectcontrol.timerBox:SetSize(150, 20)
            effectcontrol.timerBox:SetPosition(230, 20)
            effectcontrol.timerBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.timerBox:SetText(tostring(time))

            effectcontrol.effectbox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.effectbox:SetParent(effectcontrol)
            effectcontrol.effectbox:SetSize(100, 20)
            effectcontrol.effectbox:SetPosition(0, 20)
            effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectbox:SetText("Unique")
            effectcontrol.effectbox:SetChecked(highlight)

            effectcontrol.iconMatchBox = Turbine.UI.Lotro.TextBox()
            effectcontrol.iconMatchBox:SetParent(effectcontrol)
            effectcontrol.iconMatchBox:SetSize(100, 20)
            effectcontrol.iconMatchBox:SetPosition(80, 20)
            effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconMatchBox:SetText(message)

            if type == "Target_HealthThreshold" then
                effectcontrol.thresholdLabel = Turbine.UI.Label()
                effectcontrol.thresholdLabel:SetParent(effectcontrol)
                effectcontrol.thresholdLabel:SetSize(250, 20)
                effectcontrol.thresholdLabel:SetPosition(50, 0)
                effectcontrol.thresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.thresholdLabel:SetText(tostring(valuebar.browse:GetText()))

                effectcontrol.targetthresholdLabel = Turbine.UI.Label()
                effectcontrol.targetthresholdLabel:SetParent(effectcontrol)
                effectcontrol.targetthresholdLabel:SetSize(250, 20)
                effectcontrol.targetthresholdLabel:SetPosition(150, 0)
                effectcontrol.targetthresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.targetthresholdLabel:SetText(tostring(valuebar.targetNumberBar:GetText()))
            end


            if type == "Target_HealthThreshold" then
                effectcontrol.iconMatchBox.TextChanged = function()
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText()), ["Number"] = thesholdNumber, ["Operator"] = thesholdOperator})
                end


                effectcontrol.effectbox.CheckedChanged = function()
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] = icon, ["Time"] = time, ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText()), ["Number"] = thesholdNumber, ["Operator"] = thesholdOperator})
                end
            else
                effectcontrol.iconMatchBox.TextChanged = function()
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText())})
                end


                effectcontrol.effectbox.CheckedChanged = function()
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] = icon, ["Time"] = time, ["Unique"] = effectcontrol.effectbox:IsChecked(), ["Message"] = tostring(effectcontrol.iconMatchBox:GetText())})
                end
            end

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                valuebar.listbox:RemoveItem(effectcontrol)
            end

            listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
    end
end

function PrimeUIOptions:SkillBarInput(valuebar, listbox, currentValue, width, func, type)
    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 100)

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(width, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        effectcontrol.xBox = Turbine.UI.Label()
        effectcontrol.xBox:SetParent(effectcontrol)
        effectcontrol.xBox:SetSize(50, 20)
        effectcontrol.xBox:SetPosition(150, 20)
        effectcontrol.xBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.xBox:SetText("X: " .. tostring(value["XPrio"]))

        effectcontrol.yBox = Turbine.UI.Label()
        effectcontrol.yBox:SetParent(effectcontrol)
        effectcontrol.yBox:SetSize(50, 20)
        effectcontrol.yBox:SetPosition(250, 20)
        effectcontrol.yBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.yBox:SetText("Y: " .. tostring(value["YPrio"]))

        effectcontrol.iconBox = Turbine.UI.Label()
        effectcontrol.iconBox:SetParent(effectcontrol)
        effectcontrol.iconBox:SetSize(150, 20)
        effectcontrol.iconBox:SetPosition(180, 40)
        effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconBox:SetText("Time: " .. tostring(value["Time"]))

        effectcontrol.effectbox = Turbine.UI.Label()
        effectcontrol.effectbox:SetParent(effectcontrol)
        effectcontrol.effectbox:SetSize(150, 20)
        effectcontrol.effectbox:SetPosition(0, 20)
        effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectbox:SetText("Highlight: " .. tostring(value["Highlight"]))

        effectcontrol.visibitilityThreshold = Turbine.UI.Label()
        effectcontrol.visibitilityThreshold:SetParent(effectcontrol)
        effectcontrol.visibitilityThreshold:SetSize(180, 20)
        effectcontrol.visibitilityThreshold:SetPosition(0, 40)
        effectcontrol.visibitilityThreshold:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.visibitilityThreshold:SetText("Visibility T: " .. tostring(value["visibitilityThreshold"]))

        effectcontrol.hideWhileOff = Turbine.UI.Label()
        effectcontrol.hideWhileOff:SetParent(effectcontrol)
        effectcontrol.hideWhileOff:SetSize(150, 20)
        effectcontrol.hideWhileOff:SetPosition(0, 60)
        effectcontrol.hideWhileOff:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.hideWhileOff:SetText("hideWhileOff: " .. tostring(value["hideWhileOff"]))

        effectcontrol.hideWhileOn = Turbine.UI.Label()
        effectcontrol.hideWhileOn:SetParent(effectcontrol)
        effectcontrol.hideWhileOn:SetSize(150, 20)
        effectcontrol.hideWhileOn:SetPosition(0, 80)
        effectcontrol.hideWhileOn:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.hideWhileOn:SetText("hideWhileOn: " .. tostring(value["hideWhileOn"]))

        effectcontrol.opacityOn = Turbine.UI.Label()
        effectcontrol.opacityOn:SetParent(effectcontrol)
        effectcontrol.opacityOn:SetSize(150, 20)
        effectcontrol.opacityOn:SetPosition(180, 60)
        effectcontrol.opacityOn:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.opacityOn:SetText("opacityOn: " .. tostring(value["opacityOn"]))

        effectcontrol.opacityOff = Turbine.UI.Label()
        effectcontrol.opacityOff:SetParent(effectcontrol)
        effectcontrol.opacityOff:SetSize(150, 20)
        effectcontrol.opacityOff:SetPosition(180, 80)
        effectcontrol.opacityOff:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.opacityOff:SetText("opacityOff: " .. tostring(value["opacityOff"]))


        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    valuebar.searchBar = Turbine.UI.Label()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(200, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)

    valuebar.timeBar = Turbine.UI.Lotro.TextBox()
    valuebar.timeBar:SetParent(valuebar)
    valuebar.timeBar:SetSize(100, 20)
    valuebar.timeBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.timeBar:SetPosition(350, 20)
    valuebar.timeBar:SetText("Timer")

    valuebar.hiddenBar = Turbine.UI.Lotro.TextBox()
    valuebar.hiddenBar:SetParent(valuebar)
    valuebar.hiddenBar:SetSize(100, 20)
    valuebar.hiddenBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.hiddenBar:SetPosition(350, 20)
    valuebar.hiddenBar:SetVisible(false)

    valuebar.browse = Turbine.UI.Lotro.Button()
    valuebar.browse:SetParent(valuebar)
    valuebar.browse:SetSize(100, 20)
    valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.browse:SetPosition(150, 0)
    valuebar.browse:SetText("Browse")

    valuebar.xBox = Turbine.UI.Label()
    valuebar.xBox:SetParent(valuebar)
    valuebar.xBox:SetSize(50, 20)
    valuebar.xBox:SetPosition(110, 40)
    valuebar.xBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.xBox:SetText("X:")

    valuebar.xBoxValue = Turbine.UI.Lotro.TextBox()
    valuebar.xBoxValue:SetParent(valuebar)
    valuebar.xBoxValue:SetSize(80, 20)
    valuebar.xBoxValue:SetPosition(140, 40)
    valuebar.xBoxValue:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)

    valuebar.yBox = Turbine.UI.Label()
    valuebar.yBox:SetParent(valuebar)
    valuebar.yBox:SetSize(50, 20)
    valuebar.yBox:SetPosition(220, 40)
    valuebar.yBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.yBox:SetText("Y:")

    valuebar.yBoxValue = Turbine.UI.Lotro.TextBox()
    valuebar.yBoxValue:SetParent(valuebar)
    valuebar.yBoxValue:SetSize(80, 20)
    valuebar.yBoxValue:SetPosition(240, 40)
    valuebar.yBoxValue:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)

    valuebar.browse.Click = function()
        if not(self.skillCodesWindow) then self.skillCodesWindow = SkillOverview2() end
        if self.skillCodesWindow then 
            self.skillCodesWindow:SetVisible(true) 
            self.skillCodesWindow:Register(valuebar.searchBar, valuebar.hiddenBar)
        end
    end

    valuebar.effectbox = Turbine.UI.Lotro.CheckBox()
    valuebar.effectbox:SetParent(valuebar)
    valuebar.effectbox:SetSize(100, 20)
    valuebar.effectbox:SetPosition(250, 0)
    valuebar.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.effectbox:SetText("Highlight")

    valuebar.timerCheckBox = Turbine.UI.Lotro.CheckBox()
    valuebar.timerCheckBox:SetParent(valuebar)
    valuebar.timerCheckBox:SetSize(200, 20)
    valuebar.timerCheckBox:SetPosition(320, 40)
    valuebar.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.timerCheckBox:SetText("Take time from skill")

    valuebar.header = Turbine.UI.Label()
    valuebar.header:SetParent(valuebar)
    valuebar.header:SetSize(width, 20)
    valuebar.header:SetPosition(0, 60)
    valuebar.header:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.header:SetText("visibitilityThreshold   opacityOn   opacityOff")

    valuebar.visibitilityThreshold = Turbine.UI.Lotro.TextBox()
    valuebar.visibitilityThreshold:SetParent(valuebar)
    valuebar.visibitilityThreshold:SetSize(80, 20)
    valuebar.visibitilityThreshold:SetPosition(0, 80)
    valuebar.visibitilityThreshold:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.visibitilityThreshold:SetText("99999")

    valuebar.opacityOn = Turbine.UI.Lotro.TextBox()
    valuebar.opacityOn:SetParent(valuebar)
    valuebar.opacityOn:SetSize(80, 20)
    valuebar.opacityOn:SetPosition(160, 80)
    valuebar.opacityOn:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.opacityOn:SetText("1")

    valuebar.opacityOff = Turbine.UI.Lotro.TextBox()
    valuebar.opacityOff:SetParent(valuebar)
    valuebar.opacityOff:SetSize(80, 20)
    valuebar.opacityOff:SetPosition(250, 80)
    valuebar.opacityOff:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.opacityOff:SetText("1")

    valuebar.hideWhileOff = Turbine.UI.Lotro.CheckBox()
    valuebar.hideWhileOff:SetParent(valuebar)
    valuebar.hideWhileOff:SetSize(200, 20)
    valuebar.hideWhileOff:SetPosition(width - 150, 60)
    valuebar.hideWhileOff:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.hideWhileOff:SetText("hideWhileOff")

    valuebar.hideWhileOn = Turbine.UI.Lotro.CheckBox()
    valuebar.hideWhileOn:SetParent(valuebar)
    valuebar.hideWhileOn:SetSize(200, 20)
    valuebar.hideWhileOn:SetPosition(width - 150, 80)
    valuebar.hideWhileOn:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.hideWhileOn:SetText("hideWhileOn")

    valuebar.addEffect.Click = function()
        local time = tonumber(tostring((valuebar.timeBar:GetText())))
        local newEffect = tostring(valuebar.searchBar:GetText())
        if valuebar.timerCheckBox and valuebar.timerCheckBox:IsChecked() then
            time = true
        end
        local xPrio = tonumber(tostring(valuebar.xBoxValue:GetText()))
        local yPrio = tonumber(tostring(valuebar.yBoxValue:GetText()))

        local visibitilityThreshold = tonumber(tostring(valuebar.visibitilityThreshold:GetText()))
        local opacityOn = tonumber(tostring(valuebar.opacityOn:GetText()))
        local opacityOff = tonumber(tostring(valuebar.opacityOff:GetText()))
        local hideWhileOff = valuebar.hideWhileOff:IsChecked()
        local hideWhileOn = valuebar.hideWhileOn:IsChecked()

        if not(time) or not(newEffect) or not(xPrio) or not(yPrio) then ChatOutput("Not all fields are correctly filled!"); return end
        local code = tostring(valuebar.hiddenBar:GetText())
        local highlight = valuebar.effectbox:IsChecked()
        if func(newEffect, {["Time"] = time, ["Highlight"] = highlight, ["Code"] = code, ["XPrio"] = xPrio, ["YPrio"] = yPrio, 
        ["visibitilityThreshold"] = visibitilityThreshold or 99999, ["opacityOn"] = opacityOn or 1, ["opacityOff"] = opacityOff or 1, ["hideWhileOff"] = hideWhileOff, ["hideWhileOn"] = hideWhileOn}) then
            return
        end
        if newEffect then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 100)

            effectcontrol.xBox = Turbine.UI.Label()
            effectcontrol.xBox:SetParent(effectcontrol)
            effectcontrol.xBox:SetSize(150, 20)
            effectcontrol.xBox:SetPosition(150, 20)
            effectcontrol.xBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.xBox:SetText("X: " .. tostring(xPrio))
    
    
            effectcontrol.yBox = Turbine.UI.Label()
            effectcontrol.yBox:SetParent(effectcontrol)
            effectcontrol.yBox:SetSize(150, 20)
            effectcontrol.yBox:SetPosition(250, 20)
            effectcontrol.yBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.yBox:SetText("Y: " .. tostring(yPrio))

            effectcontrol.iconBox = Turbine.UI.Label()
            effectcontrol.iconBox:SetParent(effectcontrol)
            effectcontrol.iconBox:SetSize(150, 20)
            effectcontrol.iconBox:SetPosition(180, 40)
            effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconBox:SetText("Time: " .. tostring(time))

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(width, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            effectcontrol.effectbox = Turbine.UI.Label()
            effectcontrol.effectbox:SetParent(effectcontrol)
            effectcontrol.effectbox:SetSize(150, 20)
            effectcontrol.effectbox:SetPosition(0, 20)
            effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectbox:SetText("Highlight: " .. tostring(highlight))
    
            effectcontrol.visibitilityThreshold = Turbine.UI.Label()
            effectcontrol.visibitilityThreshold:SetParent(effectcontrol)
            effectcontrol.visibitilityThreshold:SetSize(180, 20)
            effectcontrol.visibitilityThreshold:SetPosition(0, 40)
            effectcontrol.visibitilityThreshold:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.visibitilityThreshold:SetText("Visibility T: " .. tostring(visibitilityThreshold))
    
            effectcontrol.hideWhileOff = Turbine.UI.Label()
            effectcontrol.hideWhileOff:SetParent(effectcontrol)
            effectcontrol.hideWhileOff:SetSize(150, 20)
            effectcontrol.hideWhileOff:SetPosition(0, 60)
            effectcontrol.hideWhileOff:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.hideWhileOff:SetText("hideWhileOff: " .. tostring(hideWhileOff))
    
            effectcontrol.hideWhileOn = Turbine.UI.Label()
            effectcontrol.hideWhileOn:SetParent(effectcontrol)
            effectcontrol.hideWhileOn:SetSize(150, 20)
            effectcontrol.hideWhileOn:SetPosition(0, 80)
            effectcontrol.hideWhileOn:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.hideWhileOn:SetText("hideWhileOn: " .. tostring(hideWhileOn))
    
            effectcontrol.opacityOn = Turbine.UI.Label()
            effectcontrol.opacityOn:SetParent(effectcontrol)
            effectcontrol.opacityOn:SetSize(150, 20)
            effectcontrol.opacityOn:SetPosition(180, 60)
            effectcontrol.opacityOn:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.opacityOn:SetText("opacityOn: " .. tostring(opacityOn))
    
            effectcontrol.opacityOff = Turbine.UI.Label()
            effectcontrol.opacityOff:SetParent(effectcontrol)
            effectcontrol.opacityOff:SetSize(150, 20)
            effectcontrol.opacityOff:SetPosition(180, 80)
            effectcontrol.opacityOff:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.opacityOff:SetText("opacityOff: " .. tostring(opacityOff))

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                valuebar.listbox:RemoveItem(effectcontrol)
            end

            listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
    end
end

function PrimeUIOptions:BuffBarInput(valuebar, listbox, currentValue, width, func, type)
    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 40)

        effectcontrol.effectlabelInfo = Turbine.UI.Label()
        effectcontrol.effectlabelInfo:SetParent(effectcontrol)
        effectcontrol.effectlabelInfo:SetSize(50, 20)
        effectcontrol.effectlabelInfo:SetPosition(180, 0)
        effectcontrol.effectlabelInfo:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabelInfo:SetText("Icon:")

        effectcontrol.iconBox = Turbine.UI.Label()
        effectcontrol.iconBox:SetParent(effectcontrol)
        effectcontrol.iconBox:SetSize(50, 20)
        effectcontrol.iconBox:SetPosition(180, 20)
        effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconBox:SetText("Time:")

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(250, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        effectcontrol.iconBox = Turbine.UI.Label()
        effectcontrol.iconBox:SetParent(effectcontrol)
        effectcontrol.iconBox:SetSize(150, 20)
        effectcontrol.iconBox:SetPosition(230, 0)
        effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconBox:SetText(tostring(value["Icon"]))

        effectcontrol.timerBox = Turbine.UI.Label()
        effectcontrol.timerBox:SetParent(effectcontrol)
        effectcontrol.timerBox:SetSize(150, 20)
        effectcontrol.timerBox:SetPosition(230, 20)
        effectcontrol.timerBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.timerBox:SetText(tostring(value["Time"]))

        effectcontrol.effectbox = Turbine.UI.Lotro.CheckBox()
        effectcontrol.effectbox:SetParent(effectcontrol)
        effectcontrol.effectbox:SetSize(100, 20)
        effectcontrol.effectbox:SetPosition(0, 20)
        effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectbox:SetText("Highlight")
        effectcontrol.effectbox:SetChecked(value["Highlight"])

        if type ==  "EffectAdded_Target" then
      
            effectcontrol.iconMatchBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.iconMatchBox:SetParent(effectcontrol)
            effectcontrol.iconMatchBox:SetSize(100, 20)
            effectcontrol.iconMatchBox:SetPosition(80, 20)
            effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconMatchBox:SetText("Match Icon")
            effectcontrol.iconMatchBox:SetChecked(value["Fit_Icon"])

            
            effectcontrol.UniqueBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.UniqueBox:SetParent(effectcontrol)
            effectcontrol.UniqueBox:SetSize(100, 20)
            effectcontrol.UniqueBox:SetPosition(280, 20)
            effectcontrol.UniqueBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.UniqueBox:SetText("Unique")
            effectcontrol.UniqueBox:SetChecked(value["Unique"])

            effectcontrol.iconMatchBox.CheckedChanged = function()
                local match = false
                if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                func(key, nil)
                func(key, {["Icon"] =  value["Icon"], ["Time"] = value["Time"], ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked(), ["Unique"] = effectcontrol.UniqueBox:IsChecked()})
            end

            effectcontrol.iconMatchBox.CheckedChanged = function()
                local match = false
                if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                func(key, nil)
                func(key, {["Icon"] =  value["Icon"], ["Time"] = value["Time"], ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked(), ["Unique"] = effectcontrol.UniqueBox:IsChecked()})
            end

        elseif type == "EffectAdded" or type == "EffectAdded_Group" or type == "EffectRemoved" or type == "Effects" then
            effectcontrol.iconMatchBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.iconMatchBox:SetParent(effectcontrol)
            effectcontrol.iconMatchBox:SetSize(100, 20)
            effectcontrol.iconMatchBox:SetPosition(80, 20)
            effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconMatchBox:SetText("Match Icon")
            effectcontrol.iconMatchBox:SetChecked(value["Fit_Icon"])

            effectcontrol.iconMatchBox.CheckedChanged = function()
                local match = false
                if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                func(key, nil)
                func(key, {["Icon"] =  value["Icon"], ["Time"] = value["Time"], ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked()})
            end
        end

        effectcontrol.effectbox.CheckedChanged = function()
            local match = false
            if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
            func(key, nil)
            func(key, {["Icon"] = value["Icon"], ["Time"] = value["Time"], ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = value["Fit_Icon"]})
        end
        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(200, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText("Name")

    valuebar.timeBar = Turbine.UI.Lotro.TextBox()
    valuebar.timeBar:SetParent(valuebar)
    valuebar.timeBar:SetSize(100, 20)
    valuebar.timeBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.timeBar:SetPosition(350, 20)
    valuebar.timeBar:SetText("Timer")

    if type == "EffectAdded" or type == "EffectAdded_Group" or type ==  "EffectAdded_Target" or type == "EffectRemoved" or type == "Effects" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.effectWindow) then self.effectWindow = HistoricalEffectOverview() end
            if self.effectWindow then 
                self.effectWindow:SetVisible(true) 
                self.effectWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "Skills" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.skillWindow) then self.skillWindow = SkillOverview() end
            if self.skillWindow then 
                self.skillWindow:SetVisible(true) 
                self.skillWindow:Register(valuebar.searchBar)
            end
        end
    end

    valuebar.EffectTimerIconBox = Turbine.UI.Control()
    valuebar.EffectTimerIconBox:SetParent(valuebar)
    valuebar.EffectTimerIconBox:SetPosition(150, 60)
    valuebar.EffectTimerIconBox:SetSize(32,32)
    valuebar.EffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
    valuebar.EffectTimerIconBox:SetVisible(true)

    valuebar.TimerIconTextBox = Turbine.UI.Lotro.TextBox()
    valuebar.TimerIconTextBox:SetParent(valuebar)
    valuebar.TimerIconTextBox:SetSize(110, 30)
    valuebar.TimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.TimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.TimerIconTextBox:SetMouseVisible(false)
    valuebar.TimerIconTextBox:SetPosition(190, 60)
    valuebar.TimerIconTextBox:SetMultiline(false)
    valuebar.TimerIconTextBox:SetText(0x41007F41)

    valuebar.EffectTimerIconBox.MouseClick = function()
        if not(self.iconWindow) then self.iconWindow = IconOverview() end
        if self.iconWindow then 
            self.iconWindow:SetVisible(true) 
            self.iconWindow:Register(valuebar.EffectTimerIconBox, valuebar.TimerIconTextBox)
        end
    end

    valuebar.EffectTimerIconBox.MouseDoubleClick = function()
        valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
        valuebar.TimerIconTextBox:SetText(0x41007F41)
    end

    valuebar.effectbox = Turbine.UI.Lotro.CheckBox()
    valuebar.effectbox:SetParent(valuebar)
    valuebar.effectbox:SetSize(100, 20)
    valuebar.effectbox:SetPosition(250, 0)
    valuebar.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.effectbox:SetText("Highlight")

    if type == "EffectAdded" or type == "EffectAdded_Group" or type ==  "EffectAdded_Target" or type == "EffectRemoved" or type == "Effects" then
        valuebar.matchBox = Turbine.UI.Lotro.CheckBox()
        valuebar.matchBox:SetParent(valuebar)
        valuebar.matchBox:SetSize(100, 20)
        valuebar.matchBox:SetPosition(300, 60)
        valuebar.matchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.matchBox:SetText("Match Icon")
    end

    if type ==  "EffectAdded_Target" then
        valuebar.uniqueBox = Turbine.UI.Lotro.CheckBox()
        valuebar.uniqueBox:SetParent(valuebar)
        valuebar.uniqueBox:SetSize(100, 20)
        valuebar.uniqueBox:SetPosition(300, 80)
        valuebar.uniqueBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.uniqueBox:SetText("Unique")
    end


    if type == "EffectAdded" or type == "EffectAdded_Group" or type ==  "EffectAdded_Target" or type == "EffectRemoved" or type == "Effects" or type == "Skills" then
        valuebar.iconBox = Turbine.UI.Lotro.CheckBox()
        valuebar.iconBox:SetParent(valuebar)
        valuebar.iconBox:SetSize(200, 20)
        valuebar.iconBox:SetPosition(150, 40)
        valuebar.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        if type == "Skills" then
            valuebar.iconBox:SetText("Take icon from skill")
        else
            valuebar.iconBox:SetText("Take icon from effect")
        end

        valuebar.timerCheckBox = Turbine.UI.Lotro.CheckBox()
        valuebar.timerCheckBox:SetParent(valuebar)
        valuebar.timerCheckBox:SetSize(200, 20)
        valuebar.timerCheckBox:SetPosition(320, 40)
        valuebar.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        if type == "Skills" then
            valuebar.timerCheckBox:SetText("Take time from skill")
        else
            valuebar.timerCheckBox:SetText("Take time from effect")
        end
    end

    valuebar.addEffect.Click = function()
        local newEffect = tostring(valuebar.searchBar:GetText())
        local time = tonumber(tostring((valuebar.timeBar:GetText())))
        if valuebar.timerCheckBox and valuebar.timerCheckBox:IsChecked() then
            time = true
        end
        if not(newEffect) or not(time) then ChatOutput("Not all fields are correctly filled!"); return end
        local icon = tostring(valuebar.TimerIconTextBox:GetText())
        if valuebar.iconBox and valuebar.iconBox:IsChecked() then
            icon = true
        end
        local highlight = valuebar.effectbox:IsChecked()
        local uniqueValue = false
        if valuebar.uniqueBox then uniqueValue = valuebar.uniqueBox:IsChecked() end
        local match = false
        if valuebar.matchBox then match = valuebar.matchBox:IsChecked() end
        if type ==  "EffectAdded_Target" then
            if func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Highlight"] = highlight, ["Fit_Icon"] = match, ["Unique"] = uniqueValue}) then
                return
            end
        else
            if func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Highlight"] = highlight, ["Fit_Icon"] = match}) then
                return
            end
        end
        if newEffect then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 40)

            
            effectcontrol.effectlabelInfo = Turbine.UI.Label()
            effectcontrol.effectlabelInfo:SetParent(effectcontrol)
            effectcontrol.effectlabelInfo:SetSize(50, 20)
            effectcontrol.effectlabelInfo:SetPosition(180, 0)
            effectcontrol.effectlabelInfo:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabelInfo:SetText("Icon:")

            effectcontrol.iconBox = Turbine.UI.Label()
            effectcontrol.iconBox:SetParent(effectcontrol)
            effectcontrol.iconBox:SetSize(50, 20)
            effectcontrol.iconBox:SetPosition(180, 20)
            effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconBox:SetText("Time:")

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(250, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            effectcontrol.iconBox = Turbine.UI.Label()
            effectcontrol.iconBox:SetParent(effectcontrol)
            effectcontrol.iconBox:SetSize(150, 20)
            effectcontrol.iconBox:SetPosition(230, 0)
            effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconBox:SetText(tostring(icon))

            effectcontrol.timerBox = Turbine.UI.Label()
            effectcontrol.timerBox:SetParent(effectcontrol)
            effectcontrol.timerBox:SetSize(150, 20)
            effectcontrol.timerBox:SetPosition(230, 20)
            effectcontrol.timerBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.timerBox:SetText(tostring(time))

            effectcontrol.effectbox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.effectbox:SetParent(effectcontrol)
            effectcontrol.effectbox:SetSize(100, 20)
            effectcontrol.effectbox:SetPosition(0, 20)
            effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectbox:SetText("Highlight")
            effectcontrol.effectbox:SetChecked(highlight)

            if type ==  "EffectAdded_Target" then

                effectcontrol.iconMatchBox = Turbine.UI.Lotro.CheckBox()
                effectcontrol.iconMatchBox:SetParent(effectcontrol)
                effectcontrol.iconMatchBox:SetSize(100, 20)
                effectcontrol.iconMatchBox:SetPosition(80, 20)
                effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.iconMatchBox:SetText("Match Icon")
                effectcontrol.iconMatchBox:SetChecked(match)

                
                effectcontrol.UniqueBox = Turbine.UI.Lotro.CheckBox()
                effectcontrol.UniqueBox:SetParent(effectcontrol)
                effectcontrol.UniqueBox:SetSize(100, 20)
                effectcontrol.UniqueBox:SetPosition(80, 20)
                effectcontrol.UniqueBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.UniqueBox:SetText("Unique")
                effectcontrol.UniqueBox:SetChecked(uniqueValue)

                effectcontrol.iconMatchBox.CheckedChanged = function()
                    local match = false
                    if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = match, ["Unique"] = effectcontrol.UniqueBox:IsChecked()})
                end

                effectcontrol.iconMatchBox.CheckedChanged = function()
                    local match = false
                    if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = match, ["Unique"] = effectcontrol.UniqueBox:IsChecked()})
                end


            elseif type == "EffectAdded" or type == "EffectAdded_Group" or type == "EffectRemoved" or type == "Effects" then
                effectcontrol.iconMatchBox = Turbine.UI.Lotro.CheckBox()
                effectcontrol.iconMatchBox:SetParent(effectcontrol)
                effectcontrol.iconMatchBox:SetSize(100, 20)
                effectcontrol.iconMatchBox:SetPosition(80, 20)
                effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.iconMatchBox:SetText("Match Icon")
                effectcontrol.iconMatchBox:SetChecked(match)

                effectcontrol.iconMatchBox.CheckedChanged = function()
                    local match = false
                    if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                    func(newEffect, nil)
                    func(newEffect, {["Icon"] =  icon, ["Time"] = time, ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = match})
                end
            end


            effectcontrol.effectbox.CheckedChanged = function()
                local match = false
                if effectcontrol.iconMatchBox then match = effectcontrol.iconMatchBox:IsChecked() end
                func(newEffect, nil)
                func(newEffect, {["Icon"] = icon, ["Time"] = time, ["Highlight"] = effectcontrol.effectbox:IsChecked(), ["Fit_Icon"] = match})
            end

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                valuebar.listbox:RemoveItem(effectcontrol)
            end

            listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
    end
end

function PrimeUIOptions:HighlightInput(valuebar, listbox, currentValue, width, func, type)
    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 40)

        effectcontrol.effectlabelInfo = Turbine.UI.Label()
        effectcontrol.effectlabelInfo:SetParent(effectcontrol)
        effectcontrol.effectlabelInfo:SetSize(50, 20)
        effectcontrol.effectlabelInfo:SetPosition(180, 0)
        effectcontrol.effectlabelInfo:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabelInfo:SetText("Icon:")

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(250, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        effectcontrol.iconBox = Turbine.UI.Label()
        effectcontrol.iconBox:SetParent(effectcontrol)
        effectcontrol.iconBox:SetSize(150, 20)
        effectcontrol.iconBox:SetPosition(230, 0)
        effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconBox:SetText(tostring(value["Icon"]))

        effectcontrol.invertBox = Turbine.UI.Lotro.CheckBox()
        effectcontrol.invertBox:SetParent(effectcontrol)
        effectcontrol.invertBox:SetSize(100, 20)
        effectcontrol.invertBox:SetPosition(180, 20)
        effectcontrol.invertBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.invertBox:SetText("Invert Highlight")
        effectcontrol.invertBox:SetChecked(value["Invert_Highlight"])

        effectcontrol.iconMatchBox = Turbine.UI.Lotro.CheckBox()
        effectcontrol.iconMatchBox:SetParent(effectcontrol)
        effectcontrol.iconMatchBox:SetSize(100, 20)
        effectcontrol.iconMatchBox:SetPosition(80, 20)
        effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.iconMatchBox:SetText("Match Icon")
        effectcontrol.iconMatchBox:SetChecked(value["Fit_Icon"])

        effectcontrol.invertBox.CheckedChanged = function()
            func(key, nil)
            func(key, {["Icon"] =  value["Icon"], ["Invert_Highlight"] = effectcontrol.invertBox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked()})
        end

        effectcontrol.iconMatchBox.CheckedChanged = function()
            func(key, nil)
            func(key, {["Icon"] =  value["Icon"], ["Invert_Highlight"] = effectcontrol.invertBox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked()})
        end

        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(200, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText("Name")

    valuebar.browse = Turbine.UI.Lotro.Button()
    valuebar.browse:SetParent(valuebar)
    valuebar.browse:SetSize(100, 20)
    valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.browse:SetPosition(150, 0)
    valuebar.browse:SetText("Browse")

    valuebar.browse.Click = function()
        if not(self.effectWindow) then self.effectWindow = HistoricalEffectOverview() end
        if self.effectWindow then 
            self.effectWindow:SetVisible(true) 
            self.effectWindow:Register(valuebar.searchBar)
        end
    end

    valuebar.EffectTimerIconBox = Turbine.UI.Control()
    valuebar.EffectTimerIconBox:SetParent(valuebar)
    valuebar.EffectTimerIconBox:SetPosition(150, 60)
    valuebar.EffectTimerIconBox:SetSize(32,32)
    valuebar.EffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
    valuebar.EffectTimerIconBox:SetVisible(true)

    valuebar.TimerIconTextBox = Turbine.UI.Lotro.TextBox()
    valuebar.TimerIconTextBox:SetParent(valuebar)
    valuebar.TimerIconTextBox:SetSize(110, 30)
    valuebar.TimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    valuebar.TimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.TimerIconTextBox:SetMouseVisible(false)
    valuebar.TimerIconTextBox:SetPosition(190, 60)
    valuebar.TimerIconTextBox:SetMultiline(false)
    valuebar.TimerIconTextBox:SetText(0x41007F41)

    valuebar.EffectTimerIconBox.MouseClick = function()
        if not(self.iconWindow) then self.iconWindow = IconOverview() end
        if self.iconWindow then 
            self.iconWindow:SetVisible(true) 
            self.iconWindow:Register(valuebar.EffectTimerIconBox, valuebar.TimerIconTextBox)
        end
    end

    valuebar.EffectTimerIconBox.MouseDoubleClick = function()
        valuebar.EffectTimerIconBox:SetBackground(0x41007F41)
        valuebar.TimerIconTextBox:SetText(0x41007F41)
    end

    valuebar.matchBox = Turbine.UI.Lotro.CheckBox()
    valuebar.matchBox:SetParent(valuebar)
    valuebar.matchBox:SetSize(100, 20)
    valuebar.matchBox:SetPosition(300, 60)
    valuebar.matchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.matchBox:SetText("Match Icon")

    valuebar.invertBox = Turbine.UI.Lotro.CheckBox()
    valuebar.invertBox:SetParent(valuebar)
    valuebar.invertBox:SetSize(200, 20)
    valuebar.invertBox:SetPosition(260, 40)
    valuebar.invertBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.invertBox:SetText("Invert Highlight")

    valuebar.addEffect.Click = function()
        local newEffect = tostring(valuebar.searchBar:GetText())
        if not(newEffect) then ChatOutput("Not all fields are correctly filled!"); return end
        local icon = tostring(valuebar.TimerIconTextBox:GetText())
        local match = false
        local invert = valuebar.invertBox:IsChecked()
        if valuebar.matchBox then match = valuebar.matchBox:IsChecked() end
        if func(newEffect, {["Icon"] =  icon, ["Invert_Highlight"] = invert, ["Fit_Icon"] = match}) then
            return
        end
        if newEffect then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 40)

            
            effectcontrol.effectlabelInfo = Turbine.UI.Label()
            effectcontrol.effectlabelInfo:SetParent(effectcontrol)
            effectcontrol.effectlabelInfo:SetSize(50, 20)
            effectcontrol.effectlabelInfo:SetPosition(180, 0)
            effectcontrol.effectlabelInfo:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabelInfo:SetText("Icon:")

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(250, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            effectcontrol.iconBox = Turbine.UI.Label()
            effectcontrol.iconBox:SetParent(effectcontrol)
            effectcontrol.iconBox:SetSize(150, 20)
            effectcontrol.iconBox:SetPosition(230, 0)
            effectcontrol.iconBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconBox:SetText(tostring(icon))

            effectcontrol.iconMatchBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.iconMatchBox:SetParent(effectcontrol)
            effectcontrol.iconMatchBox:SetSize(100, 20)
            effectcontrol.iconMatchBox:SetPosition(80, 20)
            effectcontrol.iconMatchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.iconMatchBox:SetText("Match Icon")
            effectcontrol.iconMatchBox:SetChecked(match)

            effectcontrol.invertBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.invertBox:SetParent(effectcontrol)
            effectcontrol.invertBox:SetSize(100, 20)
            effectcontrol.invertBox:SetPosition(180, 20)
            effectcontrol.invertBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.invertBox:SetText("Invert Highlight")
            effectcontrol.invertBox:SetChecked(invert)

            effectcontrol.iconMatchBox.CheckedChanged = function()
                func(newEffect, nil)
                func(newEffect, {["Icon"] =  icon, ["Invert_Highlight"] = effectcontrol.invertBox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked()})
            end

            effectcontrol.invertBox.CheckedChanged = function()
                func(key, nil)
                func(key, {["Icon"] =  icon, ["Invert_Highlight"] = effectcontrol.invertBox:IsChecked(), ["Fit_Icon"] = effectcontrol.iconMatchBox:IsChecked()})
            end

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                valuebar.listbox:RemoveItem(effectcontrol)
            end

            listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
    end
end

function PrimeUIOptions:SoundListboxInput(valuebar, listbox, currentValue, width, func, type)

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(200, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText("Name")

    if type == "Target_HealthThreshold" then
        valuebar.targetNumberBar = Turbine.UI.Lotro.TextBox()
        valuebar.targetNumberBar:SetParent(valuebar)
        valuebar.targetNumberBar:SetSize(100, 20)
        valuebar.targetNumberBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.targetNumberBar:SetPosition(350, 20)
        valuebar.targetNumberBar:SetText("Threshold")
    end

    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 40)

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(250, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        if type == "HealthThreshold" or type == "Target_HealthThreshold" then
            effectcontrol.thresholdLabel = Turbine.UI.Label()
            effectcontrol.thresholdLabel:SetParent(effectcontrol)
            effectcontrol.thresholdLabel:SetSize(250, 20)
            effectcontrol.thresholdLabel:SetPosition(50, 0)
            effectcontrol.thresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.thresholdLabel:SetText(value["Operator"])
        end

        if type == "Target_HealthThreshold" then
            effectcontrol.numberLabel = Turbine.UI.Label()
            effectcontrol.numberLabel:SetParent(effectcontrol)
            effectcontrol.numberLabel:SetSize(250, 20)
            effectcontrol.numberLabel:SetPosition(100, 0)
            effectcontrol.numberLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.numberLabel:SetText(value["Number"])
        end

        effectcontrol.valueLabel = Turbine.UI.Label()
        effectcontrol.valueLabel:SetParent(effectcontrol)
        effectcontrol.valueLabel:SetSize(250, 20)
        effectcontrol.valueLabel:SetPosition(250, 0)
        effectcontrol.valueLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        if type == "HealthThreshold" or type == "Target_HealthThreshold" then
            effectcontrol.valueLabel:SetText(value["Sound"])
        else
            effectcontrol.valueLabel:SetText(value)
        end        

        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        valuebar.listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    if type == "EffectAdded" or type == "EffectRemoved" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.effectWindow) then self.effectWindow = HistoricalEffectOverview() end
            if self.effectWindow then 
                self.effectWindow:SetVisible(true) 
                self.effectWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "Skills" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.skillWindow) then self.skillWindow = SkillOverview() end
            if self.skillWindow then 
                self.skillWindow:SetVisible(true) 
                self.skillWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "HealthThreshold" or type == "Target_HealthThreshold" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("<=")

        valuebar.browseMenu = Turbine.UI.ContextMenu()
        valuebar.browseMenuItems = valuebar.browseMenu :GetItems()

        local menuItem1 = Turbine.UI.MenuItem("<=")
        valuebar.browseMenuItems:Add(menuItem1)
        menuItem1.Click = function()
            valuebar.browse:SetText("<=")
        end

        local menuItem2 = Turbine.UI.MenuItem(">=")
        valuebar.browseMenuItems:Add(menuItem2)
        menuItem2.Click = function()
            valuebar.browse:SetText(">=")
        end
    
        valuebar.browse.Click = function()
            valuebar.browseMenu:ShowMenu()
        end
    end

    valuebar.soundButton = Turbine.UI.Lotro.Button()
    valuebar.soundButton:SetParent(valuebar)
    valuebar.soundButton:SetSize(100, 20)
    valuebar.soundButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.soundButton:SetPosition(250, 0)
    valuebar.soundButton:SetText("SoundEffect")


    valuebar.soundMenu = Turbine.UI.ContextMenu()
    valuebar.soundMenuItems = valuebar.soundMenu :GetItems()
    for key, value in spairs(SoundEffects, function(t, a, b) return a < b end) do
        local menuItem = Turbine.UI.MenuItem( key )
        valuebar.soundMenuItems:Add(menuItem)
        menuItem.Click = function()
            valuebar.soundButton:SetText(key)
        end
    end

    valuebar.soundButton.Click = function()
        valuebar.soundMenu:ShowMenu()
    end

    valuebar.addEffect.Click = function()
        local newEffect = tostring(valuebar.searchBar:GetText())
        local newSound = tostring(valuebar.soundButton:GetText())
        if type == "HealthThreshold" then
            newEffect = tonumber(newEffect)
        end
        local thresholdValue = ""
        if type == "Target_HealthThreshold" then
            newEffect = tostring(newEffect)
            thresholdValue = tonumber(tostring(valuebar.targetNumberBar:GetText()))
        end
        if not(newEffect) or newSound == "SoundEffect" or not(newSound) or  type == "Target_HealthThreshold" and not(thresholdValue) then ChatOutput("Not all fields are correctly filled!"); return end
        if type == "HealthThreshold" then
            if func(tonumber(tostring(newEffect)), {["Operator"] = tostring(valuebar.browse:GetText()), ["Sound"] = newSound}) then
                return
            end
        elseif type == "Target_HealthThreshold" then
            if func(tostring(newEffect), {["Operator"] = tostring(valuebar.browse:GetText()), ["Sound"] = newSound, ["Number"] = tonumber(valuebar.targetNumberBar:GetText()) }) then
                return
            end
        else
            if func(newEffect, newSound) then
                return
            end
        end
        if newEffect and newSound and newSound ~= "SoundEffect" then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 40)

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(250, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            if type == "HealthThreshold" or type == "Target_HealthThreshold" then
                effectcontrol.thresholdLabel = Turbine.UI.Label()
                effectcontrol.thresholdLabel:SetParent(effectcontrol)
                effectcontrol.thresholdLabel:SetSize(250, 20)
                effectcontrol.thresholdLabel:SetPosition(50, 0)
                effectcontrol.thresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.thresholdLabel:SetText(tostring(valuebar.browse:GetText()))
            end

            if type == "Target_HealthThreshold" then
                effectcontrol.targetthresholdLabel = Turbine.UI.Label()
                effectcontrol.targetthresholdLabel:SetParent(effectcontrol)
                effectcontrol.targetthresholdLabel:SetSize(250, 20)
                effectcontrol.targetthresholdLabel:SetPosition(150, 0)
                effectcontrol.targetthresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.targetthresholdLabel:SetText(tostring(valuebar.targetNumberBar:GetText()))
            end

            effectcontrol.valueLabel = Turbine.UI.Label()
            effectcontrol.valueLabel:SetParent(effectcontrol)
            effectcontrol.valueLabel:SetSize(250, 20)
            effectcontrol.valueLabel:SetPosition(250, 0)
            effectcontrol.valueLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.valueLabel:SetText(newSound) 

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                valuebar.listbox:RemoveItem(effectcontrol)
            end

            valuebar.listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
    end
end

function PrimeUIOptions:BoolListboxInput(valuebar, listbox, currentValue, width, func, type)
    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 40)

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(250, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        if type == "HealthThreshold" then
            effectcontrol.thresholdLabel = Turbine.UI.Label()
            effectcontrol.thresholdLabel:SetParent(effectcontrol)
            effectcontrol.thresholdLabel:SetSize(250, 20)
            effectcontrol.thresholdLabel:SetPosition(50, 0)
            effectcontrol.thresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.thresholdLabel:SetText(value["Operator"])
        else
            effectcontrol.effectbox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.effectbox:SetParent(effectcontrol)
            effectcontrol.effectbox:SetSize(200, 20)
            effectcontrol.effectbox:SetPosition(230, 0)
            effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectbox:SetText("Show when applied")
            effectcontrol.effectbox:SetChecked(value)

            effectcontrol.effectbox.CheckedChanged = function()
                func(key, effectcontrol.effectbox:IsChecked())
            end
        end     


        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        valuebar.listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(300, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText("Name")

    if type == "EffectAdded" or type == "EffectRemoved"or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.effectWindow) then self.effectWindow = HistoricalEffectOverview() end
            if self.effectWindow then 
                self.effectWindow:SetVisible(true) 
                self.effectWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "Skills" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.skillWindow) then self.skillWindow = SkillOverview() end
            if self.skillWindow then 
                self.skillWindow:SetVisible(true) 
                self.skillWindow:Register(valuebar.searchBar)
            end
        end
    elseif type == "HealthThreshold" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("<=")

        valuebar.browseMenu = Turbine.UI.ContextMenu()
        valuebar.browseMenuItems = valuebar.browseMenu :GetItems()

        local menuItem1 = Turbine.UI.MenuItem("<=")
        valuebar.browseMenuItems:Add(menuItem1)
        menuItem1.Click = function()
            valuebar.browse:SetText("<=")
        end

        local menuItem2 = Turbine.UI.MenuItem(">=")
        valuebar.browseMenuItems:Add(menuItem2)
        menuItem2.Click = function()
            valuebar.browse:SetText(">=")
        end
    
        valuebar.browse.Click = function()
            valuebar.browseMenu:ShowMenu()
        end
    end

    valuebar.addEffect.Click = function()
        local newEffect = tostring(valuebar.searchBar:GetText())
        if type == "HealthThreshold" then
            newEffect = tonumber(newEffect)
        end
        if not(newEffect) then ChatOutput("Not all fields are correctly filled!"); return end
        if type == "HealthThreshold" then
            if func(tonumber(tostring(newEffect)), {["Operator"] = tostring(valuebar.browse:GetText())}) then
                return
            end
        else
            if func(newEffect, true) then
                return
            end
        end

        if newEffect then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 40)

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(250, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            if type == "HealthThreshold" then
                effectcontrol.thresholdLabel = Turbine.UI.Label()
                effectcontrol.thresholdLabel:SetParent(effectcontrol)
                effectcontrol.thresholdLabel:SetSize(250, 20)
                effectcontrol.thresholdLabel:SetPosition(50, 0)
                effectcontrol.thresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.thresholdLabel:SetText(tostring(valuebar.browse:GetText()))
            else
                effectcontrol.effectbox = Turbine.UI.Lotro.CheckBox()
                effectcontrol.effectbox:SetParent(effectcontrol)
                effectcontrol.effectbox:SetSize(200, 20)
                effectcontrol.effectbox:SetPosition(230, 0)
                effectcontrol.effectbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.effectbox:SetText("Show when applied")
                effectcontrol.effectbox:SetChecked(true)
    
                effectcontrol.effectbox.CheckedChanged = function()
                    func(newEffect, nil)
                    func(newEffect, effectcontrol.effectbox:IsChecked())
                end
            end

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                valuebar.listbox:RemoveItem(effectcontrol)
            end

            valuebar.listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
    end
end

function PrimeUIOptions:NumberInput(valuebar, listbox, currentValue, width, func, type)
    for key, value in spairs(currentValue, function(t, a, b) return a < b end) do
        local effectcontrol = Turbine.UI.Label()
        effectcontrol:SetSize(width, 40)

        effectcontrol.effectlabel = Turbine.UI.Label()
        effectcontrol.effectlabel:SetParent(effectcontrol)
        effectcontrol.effectlabel:SetSize(250, 20)
        effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        effectcontrol.effectlabel:SetText(key)

        effectcontrol.timerBox = Turbine.UI.Lotro.TextBox()
        effectcontrol.timerBox:SetParent(effectcontrol)
        effectcontrol.timerBox:SetSize(150, 20)
        effectcontrol.timerBox:SetPosition(230, 0)
        effectcontrol.timerBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        if type == "Target_HealthThreshold" then
            effectcontrol.timerBox:SetText(value["Countdown"])
        else
            effectcontrol.timerBox:SetText(value)
        end

        if type == "EffectAdded" or type == "EffectRemoved" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" then
            effectcontrol.timerCheckBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.timerCheckBox:SetParent(effectcontrol)
            effectcontrol.timerCheckBox:SetSize(200, 20)
            effectcontrol.timerCheckBox:SetPosition(150, 20)
            effectcontrol.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.timerCheckBox:SetText("Take time from effect")
            effectcontrol.timerCheckBox:SetChecked(value == true)
        end

        if type == "Skills" then
            effectcontrol.timerCheckBox = Turbine.UI.Lotro.CheckBox()
            effectcontrol.timerCheckBox:SetParent(effectcontrol)
            effectcontrol.timerCheckBox:SetSize(200, 20)
            effectcontrol.timerCheckBox:SetPosition(150, 20)
            effectcontrol.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.timerCheckBox:SetText("Take time from skill")
            effectcontrol.timerCheckBox:SetChecked(value == true)
        end

        if type == "Target_HealthThreshold" then
            effectcontrol.thresholdLabel = Turbine.UI.Label()
            effectcontrol.thresholdLabel:SetParent(effectcontrol)
            effectcontrol.thresholdLabel:SetSize(250, 20)
            effectcontrol.thresholdLabel:SetPosition(200, 0)
            effectcontrol.thresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.thresholdLabel:SetText(value["Number"])

            effectcontrol.operatorLabel = Turbine.UI.Label()
            effectcontrol.operatorLabel:SetParent(effectcontrol)
            effectcontrol.operatorLabel:SetSize(250, 20)
            effectcontrol.operatorLabel:SetPosition(150, 0)
            effectcontrol.operatorLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.operatorLabel:SetText(value["Operator"])
        end

        if type == "Target_HealthThreshold" then
            effectcontrol.timerBox.TextChanged = function()
                func(key, {["Number"] = tonumber(tostring(effectcontrol.timerBox:GetText())), ["Operator"] = value["Operator"], ["Countdown"] = value["Countdown"]})
            end
        elseif type == "EffectAdded" or type == "EffectRemoved" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" or type == "Skills" then
            effectcontrol.timerBox.TextChanged = function()
                local number = tonumber(tostring(effectcontrol.timerBox:GetText()))
                if valuebar.timerCheckBox and effectcontrol.timerCheckBox:IsChecked() then
                    number = true
                end
                func(key, number)
            end

            effectcontrol.timerCheckBox.CheckedChanged = function()
                local number = tonumber(tostring(effectcontrol.timerBox:GetText()))
                if valuebar.timerCheckBox and effectcontrol.timerCheckBox:IsChecked() then
                    number = true
                end
                func(key, number)
            end
        else
            effectcontrol.timerBox.TextChanged = function()
                func(key, tonumber(tostring(effectcontrol.timerBox:GetText())))
            end
        end

        effectcontrol.Button = Turbine.UI.Lotro.Button()
        effectcontrol.Button:SetParent(effectcontrol)
        effectcontrol.Button:SetSize(100, 20)
        effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        effectcontrol.Button:SetMouseVisible(false)
        effectcontrol.Button:SetPosition(width - 110, 0)
        effectcontrol.Button:SetText("Remove")

        effectcontrol.Button.Click = function()
            func(key, nil)
            valuebar.listbox:RemoveItem(effectcontrol)
        end

        listbox:AddItem(effectcontrol)
        listbox:AddItem(self:ListBoxBorder(width))
    end

    valuebar.addEffect = Turbine.UI.Lotro.Button()
    valuebar.addEffect:SetParent(valuebar)
    valuebar.addEffect:SetSize(100, 20)
    valuebar.addEffect:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addEffect:SetPosition(350, 0)
    valuebar.addEffect:SetText("Add")

    valuebar.searchBar = Turbine.UI.Lotro.TextBox()
    valuebar.searchBar:SetParent(valuebar)
    valuebar.searchBar:SetSize(200, 20)
    valuebar.searchBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.searchBar:SetPosition(150, 20)
    valuebar.searchBar:SetText("Name")

    valuebar.timeBar = Turbine.UI.Lotro.TextBox()
    valuebar.timeBar:SetParent(valuebar)
    valuebar.timeBar:SetSize(100, 20)
    valuebar.timeBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.timeBar:SetPosition(350, 20)
    valuebar.timeBar:SetText("Timer")

    if type == "EffectAdded" or type == "EffectRemoved" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.effectWindow) then self.effectWindow = HistoricalEffectOverview() end
            if self.effectWindow then 
                self.effectWindow:SetVisible(true) 
                self.effectWindow:Register(valuebar.searchBar)
            end
        end

        valuebar.timerCheckBox = Turbine.UI.Lotro.CheckBox()
        valuebar.timerCheckBox:SetParent(valuebar)
        valuebar.timerCheckBox:SetSize(200, 20)
        valuebar.timerCheckBox:SetPosition(320, 40)
        valuebar.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.timerCheckBox:SetText("Take time from effect")

    elseif type == "Skills" then
        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("Browse")

        valuebar.browse.Click = function()
            if not(self.skillWindow) then self.skillWindow = SkillOverview() end
            if self.skillWindow then 
                self.skillWindow:SetVisible(true) 
                self.skillWindow:Register(valuebar.searchBar)
            end
        end

        valuebar.timerCheckBox = Turbine.UI.Lotro.CheckBox()
        valuebar.timerCheckBox:SetParent(valuebar)
        valuebar.timerCheckBox:SetSize(200, 20)
        valuebar.timerCheckBox:SetPosition(320, 40)
        valuebar.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.timerCheckBox:SetText("Take time from skill")

    elseif type == "Target_HealthThreshold" then
        valuebar.healthThresholdBar = Turbine.UI.Lotro.TextBox()
        valuebar.healthThresholdBar:SetParent(valuebar)
        valuebar.healthThresholdBar:SetSize(100, 20)
        valuebar.healthThresholdBar:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.healthThresholdBar:SetPosition(350, 40)
        valuebar.healthThresholdBar:SetText("Threshold")

        valuebar.browse = Turbine.UI.Lotro.Button()
        valuebar.browse:SetParent(valuebar)
        valuebar.browse:SetSize(100, 20)
        valuebar.browse:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
        valuebar.browse:SetPosition(150, 0)
        valuebar.browse:SetText("<=")

        valuebar.browseMenu = Turbine.UI.ContextMenu()
        valuebar.browseMenuItems = valuebar.browseMenu :GetItems()

        local menuItem1 = Turbine.UI.MenuItem("<=")
        valuebar.browseMenuItems:Add(menuItem1)
        menuItem1.Click = function()
            valuebar.browse:SetText("<=")
        end

        local menuItem2 = Turbine.UI.MenuItem(">=")
        valuebar.browseMenuItems:Add(menuItem2)
        menuItem2.Click = function()
            valuebar.browse:SetText(">=")
        end
    
        valuebar.browse.Click = function()
            valuebar.browseMenu:ShowMenu()
        end
    end

    valuebar.addEffect.Click = function()
        local newEffect = tostring(valuebar.searchBar:GetText())
        local threshold = false
        local countdownNumber = false
        if type == "Target_HealthThreshold" then
            countdownNumber = tonumber(tostring(valuebar.browse:GetText()))
            threshold = tonumber(valuebar.healthThresholdBar:GetText())
        end

        local number = tonumber(tostring(valuebar.timeBar:GetText()))
        if valuebar.timerCheckBox and valuebar.timerCheckBox:IsChecked() then
            number = true
        end

        if not(newEffect) or not(number) or (type == "Target_HealthThreshold" and not(countdownNumber)) or (type == "Target_HealthThreshold" and not(threshold)) then ChatOutput("Not all fields are correctly filled!"); return end
        if newEffect then
            local effectcontrol = Turbine.UI.Label()
            effectcontrol:SetSize(width, 40)

            effectcontrol.effectlabel = Turbine.UI.Label()
            effectcontrol.effectlabel:SetParent(effectcontrol)
            effectcontrol.effectlabel:SetSize(250, 20)
            effectcontrol.effectlabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.effectlabel:SetText(newEffect)

            effectcontrol.timerBox = Turbine.UI.Lotro.TextBox()
            effectcontrol.timerBox:SetParent(effectcontrol)
            effectcontrol.timerBox:SetSize(150, 20)
            effectcontrol.timerBox:SetPosition(230, 0)
            effectcontrol.timerBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            effectcontrol.timerBox:SetText(tostring(valuebar.timeBar:GetText()))

            if type == "EffectAdded" or type == "EffectRemoved" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" then
                effectcontrol.timerCheckBox = Turbine.UI.Lotro.CheckBox()
                effectcontrol.timerCheckBox:SetParent(effectcontrol)
                effectcontrol.timerCheckBox:SetSize(200, 20)
                effectcontrol.timerCheckBox:SetPosition(150, 20)
                effectcontrol.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.timerCheckBox:SetText("Take time from effect")
                effectcontrol.timerCheckBox:SetChecked(number == true)
            end

            if type == "Skills" then
                effectcontrol.timerCheckBox = Turbine.UI.Lotro.CheckBox()
                effectcontrol.timerCheckBox:SetParent(effectcontrol)
                effectcontrol.timerCheckBox:SetSize(200, 20)
                effectcontrol.timerCheckBox:SetPosition(150, 20)
                effectcontrol.timerCheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.timerCheckBox:SetText("Take time from skill")
                effectcontrol.timerCheckBox:SetChecked(number == true)
            end

            if type == "Target_HealthThreshold" then
                effectcontrol.targetthresholdLabel = Turbine.UI.Label()
                effectcontrol.targetthresholdLabel:SetParent(effectcontrol)
                effectcontrol.targetthresholdLabel:SetSize(250, 20)
                effectcontrol.targetthresholdLabel:SetPosition(150, 0)
                effectcontrol.targetthresholdLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.targetthresholdLabel:SetText( tonumber(valuebar.healthThresholdBar:GetText()))

                effectcontrol.operatorLabel = Turbine.UI.Label()
                effectcontrol.operatorLabel:SetParent(effectcontrol)
                effectcontrol.operatorLabel:SetSize(250, 20)
                effectcontrol.operatorLabel:SetPosition(200, 0)
                effectcontrol.operatorLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                effectcontrol.operatorLabel:SetText( tostring(valuebar.browse:GetText()))
            end

            if type == "Target_HealthThreshold" then
                effectcontrol.timerBox.TextChanged = function()
                    func(newEffect, {["Number"] = tonumber(tostring(valuebar.targetthresholdLabel:GetText())), ["Operator"] = tostring(valuebar.browse:GetText()), ["Countdown"] = tonumber(tostring(effectcontrol.timeBar:GetText()))})
                end
            elseif type == "EffectAdded" or type == "EffectRemoved" or type == "EffectAdded_Group" or type == "EffectAdded_Target" or type == "Effects" or type == "Skills" then
                effectcontrol.timerBox.TextChanged = function()
                    local number = tonumber(tostring(effectcontrol.timerBox:GetText()))
                    if valuebar.timerCheckBox and effectcontrol.timerCheckBox:IsChecked() then
                        number = true
                    end
                    func(newEffect, number)
                end

                effectcontrol.timerCheckBox.CheckedChanged = function()
                    local number = tonumber(tostring(effectcontrol.timerBox:GetText()))
                    if valuebar.timerCheckBox and effectcontrol.timerCheckBox:IsChecked() then
                        number = true
                    end
                    func(newEffect, number)
                end
            else
                effectcontrol.timerBox.TextChanged = function()
                    func(newEffect, tonumber(tostring(effectcontrol.timerBox:GetText())))
                end
            end

            effectcontrol.Button = Turbine.UI.Lotro.Button()
            effectcontrol.Button:SetParent(effectcontrol)
            effectcontrol.Button:SetSize(100, 20)
            effectcontrol.Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
            effectcontrol.Button:SetMouseVisible(false)
            effectcontrol.Button:SetPosition(width - 110, 0)
            effectcontrol.Button:SetText("Remove")

            effectcontrol.Button.Click = function()
                func(newEffect, nil)
                listbox:RemoveItem(effectcontrol)
            end

            listbox:AddItem(effectcontrol)
            listbox:AddItem(self:ListBoxBorder(width))
            valuebar.listbox:SetVisible(true)
        end
        if type == "Target_HealthThreshold" then
            func(newEffect, {["Number"] = threshold, ["Operator"] = tostring(valuebar.browse:GetText()), ["Countdown"] = number})
        else
            func(newEffect, number)
        end
    end
end

function PrimeUIOptions:InfoButton(parent, width, text)
    local SelectButton = Turbine.UI.Control()
	SelectButton:SetParent(parent)
	SelectButton:SetSize(16, 16)
    --SelectButton:SetBackColor(GetRandomColour())
    SelectButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SelectButton:SetBackground(0x41168DAC)
	SelectButton:SetPosition(width - 16, 5)
    SelectButton.Text = text
    SelectButton.MouseEnter = function()
        self.infoWindow:View(SelectButton.Text)
    end

    

    SelectButton.MouseLeave = function()
        self.infoWindow:SetVisible(false)
    end
end

IOWindow = class(Turbine.UI.Lotro.Window)
function IOWindow:Constructor(options)
    Turbine.UI.Lotro.Window.Constructor(self);
    self:SetSize(800, 800)
    self:SetPosition(math.floor(Turbine.UI.Display.GetWidth() / 2 - (self:GetWidth() / 2)), math.floor(Turbine.UI.Display.GetHeight() / 2 - (self:GetHeight()/2)))
    self:SetVisible(false)
    self:SetText("PrimeUI")

    self.TextBox = Turbine.UI.Lotro.TextBox()
    self.TextBox:SetParent(self)
    self.TextBox:SetSize(780, 720)
    self.TextBox:SetFont(Turbine.UI.Lotro.Font.Arial12)
    --self.TextBox:SetMouseVisible(true)
    self.TextBox:SetPosition(190, 15)
    self.TextBox:SetPosition(10, 40)

    self.SelectButton = Turbine.UI.Lotro.Button()
	self.SelectButton:SetParent(self)
	self.SelectButton:SetSize(100, 20)
	self.SelectButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.SelectButton:SetPosition(math.floor(self:GetWidth() / 2 - (100 / 2)), 760)
    self.SelectButton:SetText("Select All")

    self.SelectButton.Click = function()
        self.TextBox:SelectAll()
    end

    self.ImportButton = Turbine.UI.Lotro.Button()
	self.ImportButton:SetParent(self)
	self.ImportButton:SetSize(100, 20)
	self.ImportButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.ImportButton:SetPosition(math.floor(self:GetWidth() / 2 - (100 / 2)), 760)
    self.ImportButton:SetText("Import")

    self.options = options

    self.ImportButton.Click = function()
        local table = unpickle(self.TextBox:GetText():gsub("@@", "\n"))
        if self.options:Add(table) then
            self:SetVisible(false)
            ChatOutput("Imported PrimeUI: " .. table["Name"])
        else
            ChatOutput("Failed to add PrimeUI")
        end
    end
end

function IOWindow:View(data)
    self:SetVisible(true)
    self:SetZOrder(100)
    self.TextBox:SetText("")
    if data then
        --TableToString(data, title, self.TextBox)
        self.TextBox:SetText(pickle(data):gsub("\n", "@@"))
        self.ImportButton:SetVisible(false)
        self.SelectButton:SetVisible(true)
    else
        self.ImportButton:SetVisible(true)
        self.SelectButton:SetVisible(false)
    end
end

NewWindow = class(Turbine.UI.Lotro.Window)
function NewWindow:Constructor(options)
    Turbine.UI.Lotro.Window.Constructor(self);
    self:SetSize(300, 100)
    self:SetPosition(math.floor(Turbine.UI.Display.GetWidth() / 2 - (self:GetWidth() / 2)), math.floor(Turbine.UI.Display.GetHeight() / 2 - (self:GetHeight()/2)))
    self:SetVisible(false)
    self:SetText("Add")

    self.TextBox = Turbine.UI.Lotro.TextBox()
    self.TextBox:SetParent(self)
    self.TextBox:SetSize(280, 20)
    self.TextBox:SetFont(Turbine.UI.Lotro.Font.Arial12)
    self.TextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.TextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.TextBox:SetMouseVisible(false)
    self.TextBox:SetPosition(10, 40)

    self.ImportButton = Turbine.UI.Lotro.Button()
	self.ImportButton:SetParent(self)
	self.ImportButton:SetSize(100, 20)
	self.ImportButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.ImportButton:SetPosition(math.floor(self:GetWidth() - (130)), 65)
    self.ImportButton:SetText("Add")

    self.ICheckBox = Turbine.UI.Lotro.CheckBox()
	self.ICheckBox:SetParent(self)
	self.ICheckBox:SetSize(100, 20)
	self.ICheckBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
	self.ICheckBox:SetPosition(30, 65)
    self.ICheckBox:SetText("Advanced")
    self.ICheckBox:SetChecked(false)

    self.options = options

    self.ImportButton.Click = function()
        if self.options:CreateNew(tostring(self.TextBox:GetText()), self.ICheckBox:IsChecked()) then
            self:SetVisible(false)
        else
            ChatOutput("Failed to make new PrimeUI: There is already a PrimeUI with that name!")
        end
    end
end

function NewWindow:View()
    self:SetVisible(true)
    self:SetZOrder(100)
    self.TextBox:SetText("")
end

InfoWindow = class(Turbine.UI.Window)
function InfoWindow:Constructor()
    Turbine.UI.Window.Constructor(self);
    self:SetSize(400, 100)
    self:SetVisible(false)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.TextBox = Turbine.UI.Label()
    self.TextBox:SetParent(self)
    self.TextBox:SetSize(200, 20)
    self.TextBox:SetPosition(3, 3)
    self.TextBox:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    self.TextBox:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
    --self.TextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.TextBox:SetMouseVisible(false)
    self.TextBox:SetBackColor(Turbine.UI.Color(0.8, 0.125, 0.125, 0.125))
end

function InfoWindow:View(Text)
    self:SetVisible(true)
    local x, y = self:GetMousePosition()
    local oldX, oldY = self:GetPosition()
    self:SetPosition((x + oldX) - 200, y + oldY + 5)
    self:SetZOrder(1)
    self.TextBox:SetText(Text or "No info text available.")
    local length = Text and math.ceil(string.len(Text) / 70) + 1 or 1
    self:SetSize(400, length * 22)
    self.TextBox:SetSize(400 - 6, length * 22 - 6)
end